@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include <vector>
 #include <string>
 
-
 uint32_t crcSum(uint32_t crc, const void *data, size_t dataLen);
 uint32_t crcSumU16(uint32_t crc, const uint16_t *data, size_t dataLen);
 uint32_t crcSumVector2i(uint32_t crc, const Vector2i *data, size_t dataLen);
@@ -44,4 +43,39 @@ struct Sha256
 };
 Sha256 sha256Sum(void const *data, size_t dataLen);
 
+class EcKey
+{
+public:
+	typedef std::vector<uint8_t> Sig;
+	typedef std::vector<uint8_t> Key;
+	enum Privacy { Public, Private };
+
+	EcKey();
+	EcKey(EcKey const &b);
+	EcKey(EcKey &&b);
+	~EcKey();
+	EcKey &operator =(EcKey const &b);
+	EcKey &operator =(EcKey &&b);
+
+	void clear();
+
+	bool empty() const;
+	bool hasPrivate() const;
+
+	Sig sign(void const *data, size_t dataLen) const;
+	bool verify(Sig const &sig, void const *data, size_t dataLen) const;
+
+	Key toBytes(Privacy privacy) const;
+	void fromBytes(Key const &key, Privacy privacy);
+
+	static EcKey generate();
+
+private:
+	void *vKey;
+	static const int curveId;
+};
+
+std::string base64Encode(std::vector<uint8_t> const &bytes);
+std::vector<uint8_t> base64Decode(std::string const &str);
+
 #endif //_CRC_H_
