@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -33,12 +33,12 @@ extern char aCurrResDir[PATH_MAX];
 
 /** Set the current input buffer for the lexer
  */
-extern void res_set_extra(YY_EXTRA_TYPE user_defined);
+void res_set_extra(YY_EXTRA_TYPE user_defined);
 
 /* Call the yacc parser */
-extern int res_parse(void);
+int res_parse();
 
 /* Destroy the lexer */
-extern int res_lex_destroy(void);
+int res_lex_destroy();
 
 #endif
