@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -24,9 +24,9 @@
 
 #include "netplay.h"
 
-bool NETstartLogging(void);
-bool NETstopLogging(void);
-bool NETlogEntry(const char *str, UDWORD a, UDWORD b);
+bool NETstartLogging();
+bool NETstopLogging();
+WZ_DECL_NONNULL(1) bool NETlogEntry(const char *str, UDWORD a, UDWORD b);
 void NETlogPacket(uint8_t type, uint32_t size, bool received);
 
 #endif // _netlog_h
