@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -440,6 +440,8 @@ bool eventNewContext(SCRIPT_CODE *psCode, CONTEXT_RELEASE release,
 				if (!asCreateFuncs[type](psNewChunk->asVals + storeIndex))
 				{
 					debug(LOG_FATAL, "Failed to run script value init");
+					free(psNewChunk);
+					free(psContext);
 					return false;
 				}
 			}
