@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -25,18 +25,5 @@
 #define __INCLUDED_LIB_WIDGET_WIDGINT_H__
 
 #include "lib/framework/input.h"
-#include "widgbase.h"
-
-/* Set the id number for widgRunScreen to return */
-extern void widgSetReturn(W_SCREEN *psScreen, WIDGET *psWidget);
-
-/* Release a list of widgets */
-extern void widgReleaseWidgetList(WIDGET *psWidgets);
-
-/* Set the keyboard focus for the screen */
-extern void screenSetFocus(W_SCREEN *psScreen, WIDGET *psWidget);
-
-/* Clear the keyboard focus */
-extern void screenClearFocus(W_SCREEN *psScreen);
 
 #endif // __INCLUDED_LIB_WIDGET_WIDGINT_H__
