@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #ifndef __INCLUDED_SRC_CHALLENGE_H__
 #define __INCLUDED_SRC_CHALLENGE_H__
 
-#define CHALLENGE_SCORES "scores.ini"
+#define CHALLENGE_SCORES "scores.json"
 
 bool addChallenges(void);
 bool closeChallenges(void);
