@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -30,7 +30,6 @@
 #include "action.h"
 #include "component.h"
 #include "display3d.h"
-#include "e3demo.h"
 #include "effects.h"
 #include "intdisplay.h"
 #include "loop.h"
@@ -70,29 +69,18 @@ UBYTE getPlayerColour(UDWORD pl)
 	return NetPlay.players[pl].colour;
 }
 
-
-static void setMatrix(Vector3i *Position, Vector3i *Rotation, bool RotXYZ)
+static void setMatrix(const Vector3i *Position, const Vector3i *Rotation, int scale)
 {
 	pie_PerspectiveBegin();
 	pie_MatBegin();
 
 	pie_TRANSLATE(Position->x, Position->y, Position->z);
-
-	if (RotXYZ)
-	{
-		pie_MatRotX(DEG(Rotation->x));
-		pie_MatRotY(DEG(Rotation->y));
-		pie_MatRotZ(DEG(Rotation->z));
-	}
-	else
-	{
-		pie_MatRotY(DEG(Rotation->y));
-		pie_MatRotX(DEG(Rotation->x));
-		pie_MatRotZ(DEG(Rotation->z));
-	}
+	pie_MatRotX(DEG(Rotation->x));
+	pie_MatRotY(DEG(Rotation->y));
+	pie_MatRotZ(DEG(Rotation->z));
+	pie_MatScale(scale / 100.f);
 }
 
-
 static void unsetMatrix(void)
 {
 	pie_MatEnd();
@@ -100,13 +88,13 @@ static void unsetMatrix(void)
 }
 
 
-UDWORD getComponentDroidRadius(WZ_DECL_UNUSED DROID *psDroid)
+UDWORD getComponentDroidRadius(DROID *)
 {
 	return 100;
 }
 
 
-UDWORD getComponentDroidTemplateRadius(WZ_DECL_UNUSED DROID_TEMPLATE *psDroid)
+UDWORD getComponentDroidTemplateRadius(DROID_TEMPLATE *)
 {
 	return 100;
 }
@@ -119,7 +107,7 @@ UDWORD getComponentRadius(BASE_STATS *psComponent)
 	SDWORD compID;
 
 	compID = StatIsComponent(psComponent);
-	if (compID > 0)
+	if (compID >= 0)
 	{
 		StatGetComponentIMD(psComponent, compID, &ComponentIMD, &MountIMD);
 		if (ComponentIMD)
@@ -167,11 +155,6 @@ UDWORD getStructureStatSizeMax(STRUCTURE_STATS *Stats)
 	return MAX(Stats->baseWidth, Stats->baseBreadth);
 }
 
-static UDWORD getStructureHeight(STRUCTURE *psStructure)
-{
-	return (getStructureStatHeight(psStructure->pStructureType));
-}
-
 UDWORD getStructureStatHeight(STRUCTURE_STATS *psStat)
 {
 	if (psStat->pIMD[0])
@@ -182,135 +165,26 @@ UDWORD getStructureStatHeight(STRUCTURE_STATS *psStat)
 	return 0;
 }
 
-
-void displayIMDButton(iIMDShape *IMDShape, Vector3i *Rotation, Vector3i *Position, bool RotXYZ, SDWORD scale)
+void displayIMDButton(iIMDShape *IMDShape, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
-	setMatrix(Position, Rotation, RotXYZ);
-	pie_MatScale(scale / 100.f);
-
-	pie_SetFogStatus(false);
+	setMatrix(Position, Rotation, scale);
 	pie_Draw3DShape(IMDShape, 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
 	unsetMatrix();
 }
 
-
-//changed it to loop thru and draw all weapons
-void displayStructureButton(STRUCTURE *psStructure, Vector3i *rotation, Vector3i *Position, bool RotXYZ, SDWORD scale)
+static void sharedStructureButton(STRUCTURE_STATS *Stats, iIMDShape *strImd, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
-	iIMDShape *baseImd, *strImd; //*mountImd,*weaponImd;
-	iIMDShape *mountImd[STRUCT_MAXWEAPS];
-	iIMDShape *weaponImd[STRUCT_MAXWEAPS];
-	UDWORD			nWeaponStat;
-	int		i;
+	iIMDShape *baseImd, *mountImd[STRUCT_MAXWEAPS], *weaponImd[STRUCT_MAXWEAPS];
+	Vector3i pos = *Position;
 
-	/*HACK HACK HACK!
-	if its a 'tall thin (ie tower)' structure with something on the top - offset the
-	position to show the object on top*/
-	if (psStructure->pStructureType->pIMD[0]->nconnectors && scale == SMALL_STRUCT_SCALE &&
-	    getStructureHeight(psStructure) > TOWER_HEIGHT)
+	/* HACK HACK HACK!
+	if its a 'tall thin (ie tower)' structure stat with something on the top - offset the position to show the object on top */
+	if (strImd->nconnectors && scale == SMALL_STRUCT_SCALE && getStructureStatHeight(Stats) > TOWER_HEIGHT)
 	{
-		Position->y -= 20;
+		pos.y -= 20;
 	}
 
-	setMatrix(Position, rotation, RotXYZ);
-	pie_MatScale(scale / 100.f);
-
-	/* Draw the building's base first */
-	baseImd = psStructure->pStructureType->pBaseIMD;
-	if (baseImd != NULL)
-	{
-		pie_Draw3DShape(baseImd, 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
-	}
-	pie_Draw3DShape(psStructure->sDisplay.imd, 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
-	//and draw the turret
-	if (psStructure->sDisplay.imd->nconnectors)
-	{
-		for (i = 0; i < STRUCT_MAXWEAPS; i++)
-		{
-			weaponImd[i] = NULL;//weapon is gun ecm or sensor
-			mountImd[i] = NULL;
-		}
-
-		strImd = psStructure->sDisplay.imd;
-		//get an imd to draw on the connector priority is weapon, ECM, sensor
-		//check for weapon
-		for (i = 0; i < MAX(1, psStructure->numWeaps); i++)
-		{
-			if (psStructure->asWeaps[i].nStat > 0)
-			{
-				nWeaponStat = psStructure->asWeaps[i].nStat;
-				weaponImd[i] =  asWeaponStats[nWeaponStat].pIMD;
-				mountImd[i] =  asWeaponStats[nWeaponStat].pMountGraphic;
-			}
-
-			if (weaponImd[i] == NULL)
-			{
-				//check for ECM
-				if (psStructure->pStructureType->pECM != NULL)
-				{
-					weaponImd[i] =  psStructure->pStructureType->pECM->pIMD;
-					mountImd[i] =  psStructure->pStructureType->pECM->pMountGraphic;
-				}
-			}
-
-			if (weaponImd[i] == NULL)
-			{
-				//check for sensor
-				if (psStructure->pStructureType->pSensor != NULL)
-				{
-					weaponImd[i] =  psStructure->pStructureType->pSensor->pIMD;
-					mountImd[i]  =  psStructure->pStructureType->pSensor->pMountGraphic;
-				}
-			}
-		}
-
-		//draw Weapon/ECM/Sensor for structure
-		//uses 0
-		if (weaponImd[0] != NULL)
-		{
-			for (i = 0; i < MAX(1, psStructure->numWeaps); i++)
-			{
-				Rotation rot = structureGetInterpolatedWeaponRotation(psStructure, i, graphicsTime);
-
-				pie_MatBegin();
-				pie_TRANSLATE(strImd->connectors[i].x, strImd->connectors[i].z, strImd->connectors[i].y);
-				pie_MatRotY(-rot.direction);
-				if (mountImd[i] != NULL)
-				{
-					pie_Draw3DShape(mountImd[i], 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
-					if (mountImd[i]->nconnectors)
-					{
-						pie_TRANSLATE(mountImd[i]->connectors->x, mountImd[i]->connectors->z, mountImd[i]->connectors->y);
-					}
-				}
-				pie_MatRotX(rot.pitch);
-				pie_Draw3DShape(weaponImd[i], 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
-				//we have a droid weapon so do we draw a muzzle flash
-				pie_MatEnd();
-			}
-		}
-	}
-	unsetMatrix();
-}
-
-void displayStructureStatButton(STRUCTURE_STATS *Stats, Vector3i *Rotation, Vector3i *Position, bool RotXYZ, SDWORD scale)
-{
-	iIMDShape		*baseImd, *strImd; //*mountImd,*weaponImd;
-	iIMDShape *mountImd[STRUCT_MAXWEAPS];
-	iIMDShape *weaponImd[STRUCT_MAXWEAPS];
-	UBYTE	i;
-
-	/*HACK HACK HACK!
-	if its a 'tall thin (ie tower)' structure stat with something on the top - offset the
-	position to show the object on top*/
-	if (Stats->pIMD[0]->nconnectors && scale == SMALL_STRUCT_SCALE &&
-	    getStructureStatHeight(Stats) > TOWER_HEIGHT)
-	{
-		Position->y -= 20;
-	}
-
-	setMatrix(Position, Rotation, RotXYZ);
-	pie_MatScale(scale / 100.f);
+	setMatrix(&pos, Rotation, scale);
 
 	/* Draw the building's base first */
 	baseImd = Stats->pBaseIMD;
@@ -319,29 +193,22 @@ void displayStructureStatButton(STRUCTURE_STATS *Stats, Vector3i *Rotation, Vect
 	{
 		pie_Draw3DShape(baseImd, 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
 	}
-	pie_Draw3DShape(Stats->pIMD[0], 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
+	pie_Draw3DShape(strImd, 0, getPlayerColour(selectedPlayer), WZCOL_WHITE, pie_BUTTON, 0);
 
 	//and draw the turret
-	if (Stats->pIMD[0]->nconnectors)
+	if (strImd->nconnectors)
 	{
-		if (Stats->numWeaps > 0)
+		weaponImd[0] = NULL;
+		mountImd[0] = NULL;
+		for (int i = 0; i < Stats->numWeaps; i++)
 		{
-			for (i = 0; i < Stats->numWeaps; i++)
-			{
-				weaponImd[i] = NULL;//weapon is gun ecm or sensor
-				mountImd[i] = NULL;
-			}
+			weaponImd[i] = NULL;//weapon is gun ecm or sensor
+			mountImd[i] = NULL;
 		}
-		else
-		{
-			weaponImd[0] = NULL;
-			mountImd[0] = NULL;
-		}
-		strImd = Stats->pIMD[0];
 		//get an imd to draw on the connector priority is weapon, ECM, sensor
 		//check for weapon
 		//can only have the STRUCT_MAXWEAPS
-		for (i = 0; i < MAX(1, Stats->numWeaps); i++)
+		for (int i = 0; i < MAX(1, Stats->numWeaps); i++)
 		{
 			//can only have the one
 			if (Stats->psWeapStat[i] != NULL)
@@ -374,7 +241,7 @@ void displayStructureStatButton(STRUCTURE_STATS *Stats, Vector3i *Rotation, Vect
 		//draw Weapon/ECM/Sensor for structure
 		if (weaponImd[0] != NULL)
 		{
-			for (i = 0; i < MAX(1, Stats->numWeaps); i++)
+			for (int i = 0; i < MAX(1, Stats->numWeaps); i++)
 			{
 				pie_MatBegin();
 				pie_TRANSLATE(strImd->connectors[i].x, strImd->connectors[i].z, strImd->connectors[i].y);
@@ -396,30 +263,33 @@ void displayStructureStatButton(STRUCTURE_STATS *Stats, Vector3i *Rotation, Vect
 	unsetMatrix();
 }
 
+void displayStructureButton(STRUCTURE *psStructure, const Vector3i *rotation, const Vector3i *Position, int scale)
+{
+	sharedStructureButton(psStructure->pStructureType, psStructure->sDisplay.imd, rotation, Position, scale);
+}
 
+void displayStructureStatButton(STRUCTURE_STATS *Stats, const Vector3i *rotation, const Vector3i *Position, int scale)
+{
+	sharedStructureButton(Stats, Stats->pIMD[0], rotation, Position, scale);
+}
 
 // Render a component given a BASE_STATS structure.
 //
-void displayComponentButton(BASE_STATS *Stat, Vector3i *Rotation, Vector3i *Position,
-                            bool RotXYZ, SDWORD scale)
+void displayComponentButton(BASE_STATS *Stat, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	iIMDShape *ComponentIMD = NULL;
 	iIMDShape *MountIMD = NULL;
-	SDWORD compID;
+	int compID = StatIsComponent(Stat);
 
-	setMatrix(Position, Rotation, RotXYZ);
-	pie_MatScale(scale / 100.f);
-
-	compID = StatIsComponent(Stat);
-	if (compID > 0)
+	if (compID >= 0)
 	{
 		StatGetComponentIMD(Stat, compID, &ComponentIMD, &MountIMD);
 	}
 	else
 	{
-		unsetMatrix();
 		return;
 	}
+	setMatrix(Position, Rotation, scale);
 
 	/* VTOL bombs are only stats allowed to have NULL ComponentIMD */
 	if (StatIsComponent(Stat) != COMP_WEAPON
@@ -450,15 +320,15 @@ void displayComponentButton(BASE_STATS *Stat, Vector3i *Rotation, Vector3i *Posi
 
 // Render a research item given a BASE_STATS structure.
 //
-void displayResearchButton(BASE_STATS *Stat, Vector3i *Rotation, Vector3i *Position, bool RotXYZ, SDWORD scale)
+void displayResearchButton(BASE_STATS *Stat, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	iIMDShape *ResearchIMD = ((RESEARCH *)Stat)->pIMD;
 	iIMDShape *MountIMD = ((RESEARCH *)Stat)->pIMD2;
 
+	ASSERT(ResearchIMD, "ResearchIMD is NULL");
 	if (ResearchIMD)
 	{
-		setMatrix(Position, Rotation, RotXYZ);
-		pie_MatScale(scale / 100.f);
+		setMatrix(Position, Rotation, scale);
 
 		if (MountIMD)
 		{
@@ -468,55 +338,72 @@ void displayResearchButton(BASE_STATS *Stat, Vector3i *Rotation, Vector3i *Posit
 
 		unsetMatrix();
 	}
-	else
+}
+
+
+static inline iIMDShape *getLeftPropulsionIMD(DROID *psDroid)
+{
+	int bodyStat = psDroid->asBits[COMP_BODY];
+	int propStat = psDroid->asBits[COMP_PROPULSION];
+	return asBodyStats[bodyStat].ppIMDList[propStat * NUM_PROP_SIDES + LEFT_PROP];
+}
+
+static inline iIMDShape *getRightPropulsionIMD(DROID *psDroid)
+{
+	int bodyStat = psDroid->asBits[COMP_BODY];
+	int propStat = psDroid->asBits[COMP_PROPULSION];
+	return asBodyStats[bodyStat].ppIMDList[propStat * NUM_PROP_SIDES + RIGHT_PROP];
+}
+
+void drawMuzzleFlash(WEAPON sWeap, iIMDShape *weaponImd, iIMDShape *flashImd, PIELIGHT buildingBrightness, int pieFlag, int iPieData, UBYTE colour)
+{
+	if (!weaponImd || !flashImd || !weaponImd->nconnectors || graphicsTime < sWeap.lastFired)
 	{
-		debug(LOG_ERROR, "ResearchIMD == NULL");
+		return;
+	}
+
+	int connector_num = 0;
+
+	// which barrel is firing if model have multiple muzzle connectors?
+	if (sWeap.shotsFired && (weaponImd->nconnectors > 1))
+	{
+		// shoot first, draw later - substract one shot to get correct results
+		connector_num = (sWeap.shotsFired - 1) % (weaponImd->nconnectors);
+	}
+
+	/* Now we need to move to the end of the firing barrel */
+	pie_TRANSLATE(weaponImd->connectors[connector_num].x,
+	              weaponImd->connectors[connector_num].z,
+	              weaponImd->connectors[connector_num].y);
+
+	// assume no clan colours for muzzle effects
+	if (flashImd->numFrames == 0 || flashImd->animInterval <= 0)
+	{
+		// no anim so display one frame for a fixed time
+		if (graphicsTime >= sWeap.lastFired && graphicsTime < sWeap.lastFired + BASE_MUZZLE_FLASH_DURATION)
+		{
+			pie_Draw3DShape(flashImd, 0, colour, buildingBrightness, pieFlag | pie_ADDITIVE, EFFECT_MUZZLE_ADDITIVE);
+		}
+	}
+	else if (graphicsTime >= sWeap.lastFired)
+	{
+		// animated muzzle
+		int frame = (graphicsTime - sWeap.lastFired) / flashImd->animInterval;
+		if (frame < flashImd->numFrames)
+		{
+			pie_Draw3DShape(flashImd, frame, colour, buildingBrightness, pieFlag | pie_ADDITIVE, EFFECT_MUZZLE_ADDITIVE);
+		}
 	}
 }
 
-
-
-static iIMDShape *getLeftPropulsionIMD(DROID *psDroid)
-{
-	UDWORD			bodyStat, propStat;
-	iIMDShape		**imd;
-
-	bodyStat = psDroid->asBits[COMP_BODY].nStat;
-	propStat = psDroid->asBits[COMP_PROPULSION].nStat;
-
-	imd = asBodyStats[bodyStat].ppIMDList;
-	imd += (propStat * NUM_PROP_SIDES + LEFT_PROP);
-
-	return *imd;
-}
-
-
-static iIMDShape *getRightPropulsionIMD(DROID *psDroid)
-{
-	UDWORD			bodyStat, propStat;
-	iIMDShape		**imd;
-
-	bodyStat = psDroid->asBits[COMP_BODY].nStat;
-	propStat = psDroid->asBits[COMP_PROPULSION].nStat;
-
-	imd = asBodyStats[bodyStat].ppIMDList;
-	imd += (propStat * NUM_PROP_SIDES + RIGHT_PROP);
-
-	return *imd;
-}
-
-
 /* Assumes matrix context is already set */
 // this is able to handle multiple weapon graphics now
 // removed mountRotation,they get such stuff from psObj directly now
 static void displayCompObj(DROID *psDroid, bool bButton)
 {
-	iIMDShape               *psShape, *psJet, *psShapeTemp = NULL, *psMountShape;
-	Vector3i                zero(0, 0, 0);
-	Vector2i				screenCoords;
+	iIMDShape               *psShape, *psMoveAnim, *psStillAnim, *psShapeTemp = NULL, *psMountShape;
 	SDWORD				iConnector;
 	PROPULSION_STATS	*psPropStats;
-	SDWORD				frame;
 	SDWORD				pieFlag, iPieData;
 	PIELIGHT			brightness;
 	UDWORD				colour;
@@ -532,7 +419,7 @@ static void displayCompObj(DROID *psDroid, bool bButton)
 	}
 
 	/* get propulsion stats */
-	psPropStats = asPropulsionStats + psDroid->asBits[COMP_PROPULSION].nStat;
+	psPropStats = asPropulsionStats + psDroid->asBits[COMP_PROPULSION];
 	ASSERT_OR_RETURN(, psPropStats != NULL, "invalid propulsion stats pointer");
 
 	//set pieflag for button object or ingame object
@@ -557,7 +444,7 @@ static void displayCompObj(DROID *psDroid, bool bButton)
 	}
 
 	/* set default components transparent */
-	if (psDroid->asBits[COMP_PROPULSION].nStat == 0)
+	if (psDroid->asBits[COMP_PROPULSION] == 0)
 	{
 		pieFlag  |= pie_TRANSLUCENT;
 		iPieData  = DEFAULT_COMPONENT_TRANSLUCENCY;
@@ -581,7 +468,7 @@ static void displayCompObj(DROID *psDroid, bool bButton)
 	}
 
 	/* set default components transparent */
-	if (psDroid->asBits[COMP_BODY].nStat == 0)
+	if (psDroid->asBits[COMP_BODY] == 0)
 	{
 		pieFlag  |= pie_TRANSLUCENT;
 		iPieData  = DEFAULT_COMPONENT_TRANSLUCENCY;
@@ -622,24 +509,16 @@ static void displayCompObj(DROID *psDroid, bool bButton)
 		}
 	}
 
-	/* render vtol jet if flying - horrible hack - GJ */
-	if (((psPropStats->propulsionType == PROPULSION_TYPE_LIFT) &&
-	     //(psDroid->droidType != DROID_CYBORG)) && (!bButton))
-	     (!cyborgDroid(psDroid))) && (!bButton))
+	/* Render animation effects based on movement or lack thereof, if any */
+	psMoveAnim = asBodyStats[psDroid->asBits[COMP_BODY]].ppMoveIMDList[psDroid->asBits[COMP_PROPULSION]];
+	psStillAnim = asBodyStats[psDroid->asBits[COMP_BODY]].ppStillIMDList[psDroid->asBits[COMP_PROPULSION]];
+	if (!bButton && psMoveAnim && psDroid->sMove.Status != MOVEINACTIVE)
 	{
-		/* show flame if above ground */
-		if (psDroid->sMove.Status != MOVEINACTIVE)
-		{
-			/* draw flame if found  */
-
-			/* GJ TODO: add flame-finding code here */
-			psJet = asBodyStats[psDroid->asBits[COMP_BODY].nStat].pFlameIMD;
-
-			if (psJet != NULL)
-			{
-				pie_Draw3DShape(psJet, getModularScaledGraphicsTime(psJet->animInterval, psJet->numFrames), colour, brightness, pie_ADDITIVE, 200);
-			}
-		}
+		pie_Draw3DShape(psMoveAnim, getModularScaledGraphicsTime(psMoveAnim->animInterval, psMoveAnim->numFrames), colour, brightness, pie_ADDITIVE, 200);
+	}
+	else if (!bButton && psStillAnim) // standing still
+	{
+		pie_Draw3DShape(psStillAnim, getModularScaledGraphicsTime(psStillAnim->animInterval, psStillAnim->numFrames), colour, brightness, 0, 0);
 	}
 
 	//don't change the screen coords of an object if drawing it in a button
@@ -650,12 +529,12 @@ static void displayCompObj(DROID *psDroid, bool bButton)
 	}
 
 	/* set default components transparent */
-	if (psDroid->asWeaps[0].nStat              == 0 &&
-	    psDroid->asBits[COMP_SENSOR].nStat     == 0 &&
-	    psDroid->asBits[COMP_ECM].nStat        == 0 &&
-	    psDroid->asBits[COMP_BRAIN].nStat      == 0 &&
-	    psDroid->asBits[COMP_REPAIRUNIT].nStat == 0 &&
-	    psDroid->asBits[COMP_CONSTRUCT].nStat  == 0)
+	if (psDroid->asWeaps[0].nStat        == 0 &&
+	    psDroid->asBits[COMP_SENSOR]     == 0 &&
+	    psDroid->asBits[COMP_ECM]        == 0 &&
+	    psDroid->asBits[COMP_BRAIN]      == 0 &&
+	    psDroid->asBits[COMP_REPAIRUNIT] == 0 &&
+	    psDroid->asBits[COMP_CONSTRUCT]  == 0)
 	{
 		pieFlag  |= pie_TRANSLUCENT;
 		iPieData  = DEFAULT_COMPONENT_TRANSLUCENCY;
@@ -666,277 +545,229 @@ static void displayCompObj(DROID *psDroid, bool bButton)
 		iPieData = 0;
 	}
 
-	/* Indenting here is only to show new matrix context */
+	psShapeTemp = BODY_IMD(psDroid, psDroid->player);
+	if (psShapeTemp && psShapeTemp->nconnectors)
 	{
-		psShapeTemp = BODY_IMD(psDroid, psDroid->player);
-		if (psShapeTemp && psShapeTemp->nconnectors)
+		/* vtol weapons attach to connector 2 (underneath);
+		 * all others to connector 1 */
+		/* VTOL's now skip the first 5 connectors(0 to 4),
+		VTOL's use 5,6,7,8 etc now */
+		if (psPropStats->propulsionType == PROPULSION_TYPE_LIFT && psDroid->droidType == DROID_WEAPON)
 		{
-			/* vtol weapons attach to connector 2 (underneath);
-			 * all others to connector 1 */
-			/* VTOL's now skip the first 5 connectors(0 to 4),
-			VTOL's use 5,6,7,8 etc now */
-			if ((psPropStats->propulsionType == PROPULSION_TYPE_LIFT) &&
-			    psDroid->droidType == DROID_WEAPON)
-			{
-				iConnector = VTOL_CONNECTOR_START;
-			}
-			else
-			{
-				iConnector = 0;
-			}
+			iConnector = VTOL_CONNECTOR_START;
+		}
+		else
+		{
+			iConnector = 0;
+		}
 
-			switch (psDroid->droidType)
+		switch (psDroid->droidType)
+		{
+		case DROID_DEFAULT:
+		case DROID_TRANSPORTER:
+		case DROID_SUPERTRANSPORTER:
+		case DROID_CYBORG:
+		case DROID_CYBORG_SUPER:
+		case DROID_WEAPON:
+		case DROID_COMMAND:		// command droids have a weapon to store all the graphics
+			/*	Get the mounting graphic - we've already moved to the right position
+			Allegedly - all droids will have a mount graphic so this shouldn't
+			fall on it's arse......*/
+			/* Double check that the weapon droid actually has any */
+			for (i = 0; i < psDroid->numWeaps; i++)
 			{
-			case DROID_DEFAULT:
-			case DROID_TRANSPORTER:
-			case DROID_SUPERTRANSPORTER:
-			case DROID_CYBORG:
-			case DROID_CYBORG_SUPER:
-			case DROID_WEAPON:
-			case DROID_COMMAND:		// command droids have a weapon to store all the graphics
-				/*	Get the mounting graphic - we've already moved to the right position
-				Allegedly - all droids will have a mount graphic so this shouldn't
-				fall on it's arse......*/
-				/* Double check that the weapon droid actually has any */
-				for (i = 0; i < psDroid->numWeaps; i++)
+				if ((psDroid->asWeaps[i].nStat > 0 || psDroid->droidType == DROID_DEFAULT)
+				    && psShapeTemp->connectors)
 				{
-					if (psDroid->asWeaps[i].nStat > 0 || psDroid->droidType == DROID_DEFAULT)
+					Rotation rot = getInterpolatedWeaponRotation(psDroid, i, graphicsTime);
+
+					pie_MatBegin(!bButton);
+
+					//to skip number of VTOL_CONNECTOR_START ground unit connectors
+					if (iConnector < VTOL_CONNECTOR_START)
 					{
-						if (psShapeTemp->connectors)
-						{
-							Rotation rot = getInterpolatedWeaponRotation(psDroid, i, graphicsTime);
-
-							pie_MatBegin();
-
-							//to skip number of VTOL_CONNECTOR_START ground unit connectors
-							if (iConnector < VTOL_CONNECTOR_START)
-							{
-								pie_TRANSLATE(psShapeTemp->connectors[i].x,
-								              psShapeTemp->connectors[i].z,
-								              psShapeTemp->connectors[i].y);
-							}
-							else
-							{
-								pie_TRANSLATE(psShapeTemp->connectors[iConnector + i].x,
-								              psShapeTemp->connectors[iConnector + i].z,
-								              psShapeTemp->connectors[iConnector + i].y);
-							}
-
-							pie_MatRotY(-rot.direction);
-
-							/* vtol weapons inverted */
-							if (iConnector >= VTOL_CONNECTOR_START)
-							{
-								pie_MatRotZ(65536 / 2); //this might affect gun rotation
-							}
-
-							/* Get the mount graphic */
-							psShape = WEAPON_MOUNT_IMD(psDroid, i);
-
-							int recoilValue = getRecoil(psDroid->asWeaps[i]);
-							pie_TRANSLATE(0, 0, recoilValue / 3);
-
-							/* Draw it */
-							if (psShape)
-							{
-								pie_Draw3DShape(psShape, 0, colour, brightness, pieFlag, iPieData);
-							}
-
-							pie_TRANSLATE(0, 0, recoilValue);
-
-							/* translate for weapon mount point */
-							if (psShape && psShape->nconnectors)
-							{
-								pie_TRANSLATE(psShape->connectors->x, psShape->connectors->z, psShape->connectors->y);
-							}
-
-							/* vtol weapons inverted */
-							if (iConnector >= VTOL_CONNECTOR_START)
-							{
-								//pitch the barrel down
-								pie_MatRotX(-rot.pitch);
-							}
-							else
-							{
-								//pitch the barrel up
-								pie_MatRotX(rot.pitch);
-							}
-
-							/* Get the weapon (gun?) graphic */
-							psShape = WEAPON_IMD(psDroid, i);
-
-							// We have a weapon so we draw it and a muzzle flash from weapon connector
-							if (psShape)
-							{
-								pie_Draw3DShape(psShape, 0, colour, brightness, pieFlag, iPieData);
-
-								if (psShape->nconnectors)
-								{
-									unsigned int connector_num = 0;
-
-									// which barrel is firing if model have multiple muzzle connectors?
-									if (psDroid->asWeaps[i].shotsFired && (psShape->nconnectors > 1))
-									{
-										// shoot first, draw later - substract one shot to get correct results
-										connector_num = (psDroid->asWeaps[i].shotsFired - 1) % (psShape->nconnectors);
-									}
-
-									/* Now we need to move to the end of the firing barrel (there maybe multiple barrels) */
-									pie_TRANSLATE(psShape->connectors[connector_num].x,
-									              psShape->connectors[connector_num].z,
-									              psShape->connectors[connector_num].y);
-
-									//and draw the muzzle flash
-									psShape = MUZZLE_FLASH_PIE(psDroid, i);
-
-									if (psShape && graphicsTime >= psDroid->asWeaps[i].lastFired)
-									{
-										//assume no clan colours for muzzle effects
-										if ((psShape->numFrames == 0) || (psShape->animInterval <= 0))
-										{
-											//no anim so display one frame for a fixed time
-											if (graphicsTime < psDroid->asWeaps[i].lastFired + BASE_MUZZLE_FLASH_DURATION)
-											{
-												pie_Draw3DShape(psShape, 0, 0, brightness, pieFlag | pie_ADDITIVE, EFFECT_MUZZLE_ADDITIVE);
-											}
-										}
-										else
-										{
-											// animated muzzle
-											frame = (graphicsTime - psDroid->asWeaps[i].lastFired) / psShape->animInterval;
-											if (frame < psShape->numFrames)
-											{
-												pie_Draw3DShape(psShape, frame, 0, brightness, pieFlag | pie_ADDITIVE, EFFECT_MUZZLE_ADDITIVE);
-											}
-										}
-									}
-								}
-							}
-						}
-						/* Pop Matrix */
-						pie_MatEnd();
+						pie_TRANSLATE(psShapeTemp->connectors[i].x,
+						              psShapeTemp->connectors[i].z,
+						              psShapeTemp->connectors[i].y);
 					}
-				}
-				break;
-
-			case DROID_SENSOR:
-			case DROID_CONSTRUCT:
-			case DROID_CYBORG_CONSTRUCT:
-			case DROID_ECM:
-			case DROID_REPAIR:
-			case DROID_CYBORG_REPAIR:
-				{
-					Rotation rot = getInterpolatedWeaponRotation(psDroid, 0, graphicsTime);
-
-					switch (psDroid->droidType)
+					else
 					{
-					default: ASSERT(false, "...");
-					case DROID_SENSOR:
-						psMountShape = SENSOR_MOUNT_IMD(psDroid, psDroid->player);
-						/* Get the sensor graphic, assuming it's there */
-						psShape = SENSOR_IMD(psDroid, psDroid->player);
-						break;
-					case DROID_CONSTRUCT:
-					case DROID_CYBORG_CONSTRUCT:
-						psMountShape = CONSTRUCT_MOUNT_IMD(psDroid, psDroid->player);
-						/* Get the construct graphic assuming it's there */
-						psShape = CONSTRUCT_IMD(psDroid, psDroid->player);
-						break;
-					case DROID_ECM:
-						psMountShape = ECM_MOUNT_IMD(psDroid, psDroid->player);
-						/* Get the ECM graphic assuming it's there.... */
-						psShape = ECM_IMD(psDroid, psDroid->player);
-						break;
-					case DROID_REPAIR:
-					case DROID_CYBORG_REPAIR:
-						psMountShape = REPAIR_MOUNT_IMD(psDroid, psDroid->player);
-						/* Get the Repair graphic assuming it's there.... */
-						psShape = REPAIR_IMD(psDroid, psDroid->player);
-						break;
+						pie_TRANSLATE(psShapeTemp->connectors[iConnector + i].x,
+						              psShapeTemp->connectors[iConnector + i].z,
+						              psShapeTemp->connectors[iConnector + i].y);
 					}
-					/*	Get the mounting graphic - we've already moved to the right position
-					Allegedly - all droids will have a mount graphic so this shouldn't
-					fall on it's arse......*/
-					//sensor and cyborg and ecm uses connectors[0]
-					pie_MatBegin();
+
+					pie_MatRotY(-rot.direction);
+
 					/* vtol weapons inverted */
 					if (iConnector >= VTOL_CONNECTOR_START)
 					{
 						pie_MatRotZ(65536 / 2); //this might affect gun rotation
 					}
 
-					pie_TRANSLATE(psShapeTemp->connectors[0].x,
-					              psShapeTemp->connectors[0].z,
-					              psShapeTemp->connectors[0].y);
+					/* Get the mount graphic */
+					psShape = WEAPON_MOUNT_IMD(psDroid, i);
 
-					pie_MatRotY(-rot.direction);
-					/* Draw it */
-					if (psMountShape)
-					{
-						pie_Draw3DShape(psMountShape, 0, colour, brightness, pieFlag, iPieData);
-					}
-
-					/* translate for construct mount point if cyborg */
-					if (cyborgDroid(psDroid) && psMountShape && psMountShape->nconnectors)
-					{
-						pie_TRANSLATE(psMountShape->connectors[0].x,
-						              psMountShape->connectors[0].z,
-						              psMountShape->connectors[0].y);
-					}
+					int recoilValue = getRecoil(psDroid->asWeaps[i]);
+					pie_TRANSLATE(0, 0, recoilValue / 3);
 
 					/* Draw it */
 					if (psShape)
 					{
 						pie_Draw3DShape(psShape, 0, colour, brightness, pieFlag, iPieData);
+					}
 
-						// In repair droid case only:
-						if ((psDroid->droidType == DROID_REPAIR || psDroid->droidType == DROID_CYBORG_REPAIR) &&
-						    psShape->nconnectors && psDroid->action == DACTION_DROIDREPAIR)
-						{
-							Spacetime st = interpolateObjectSpacetime(psDroid, graphicsTime);
-							Rotation rot = getInterpolatedWeaponRotation(psDroid, 0, graphicsTime);
+					pie_TRANSLATE(0, 0, recoilValue);
 
-							pie_TRANSLATE(psShape->connectors[0].x,
-							              psShape->connectors[0].z,
-							              psShape->connectors[0].y);
-							pie_TRANSLATE(0, -20, 0);
+					/* translate for weapon mount point */
+					if (psShape && psShape->nconnectors)
+					{
+						pie_TRANSLATE(psShape->connectors->x, psShape->connectors->z, psShape->connectors->y);
+					}
 
-							psShape = getImdFromIndex(MI_FLAME);
+					/* vtol weapons inverted */
+					if (iConnector >= VTOL_CONNECTOR_START)
+					{
+						//pitch the barrel down
+						pie_MatRotX(-rot.pitch);
+					}
+					else
+					{
+						//pitch the barrel up
+						pie_MatRotX(rot.pitch);
+					}
 
-							/* Rotate for droid */
-							pie_MatRotY(st.rot.direction);
-							pie_MatRotX(-st.rot.pitch);
-							pie_MatRotZ(-st.rot.roll);
-							//rotate Y
-							pie_MatRotY(rot.direction);
+					/* Get the weapon (gun?) graphic */
+					psShape = WEAPON_IMD(psDroid, i);
 
-							pie_MatRotY(-player.r.y);
-							pie_MatRotX(-player.r.x);
-
-							pie_Draw3DShape(psShape, getModularScaledGraphicsTime(psShape->animInterval, psShape->numFrames), 0, brightness, pie_ADDITIVE, 140);
-
-							pie_MatRotX(player.r.x);
-							pie_MatRotY(player.r.y);
-						}
+					// We have a weapon so we draw it and a muzzle flash from weapon connector
+					if (psShape)
+					{
+						pie_Draw3DShape(psShape, 0, colour, brightness, pieFlag, iPieData);
+						drawMuzzleFlash(psDroid->asWeaps[i], psShape, MUZZLE_FLASH_PIE(psDroid, i), brightness, pieFlag, iPieData);
 					}
 					/* Pop Matrix */
 					pie_MatEnd();
+				}
+			}
+			break;
+
+		case DROID_SENSOR:
+		case DROID_CONSTRUCT:
+		case DROID_CYBORG_CONSTRUCT:
+		case DROID_ECM:
+		case DROID_REPAIR:
+		case DROID_CYBORG_REPAIR:
+			{
+				Rotation rot = getInterpolatedWeaponRotation(psDroid, 0, graphicsTime);
+
+				switch (psDroid->droidType)
+				{
+				default: ASSERT(false, "...");
+				case DROID_SENSOR:
+					psMountShape = SENSOR_MOUNT_IMD(psDroid, psDroid->player);
+					/* Get the sensor graphic, assuming it's there */
+					psShape = SENSOR_IMD(psDroid, psDroid->player);
+					break;
+				case DROID_CONSTRUCT:
+				case DROID_CYBORG_CONSTRUCT:
+					psMountShape = CONSTRUCT_MOUNT_IMD(psDroid, psDroid->player);
+					/* Get the construct graphic assuming it's there */
+					psShape = CONSTRUCT_IMD(psDroid, psDroid->player);
+					break;
+				case DROID_ECM:
+					psMountShape = ECM_MOUNT_IMD(psDroid, psDroid->player);
+					/* Get the ECM graphic assuming it's there.... */
+					psShape = ECM_IMD(psDroid, psDroid->player);
+					break;
+				case DROID_REPAIR:
+				case DROID_CYBORG_REPAIR:
+					psMountShape = REPAIR_MOUNT_IMD(psDroid, psDroid->player);
+					/* Get the Repair graphic assuming it's there.... */
+					psShape = REPAIR_IMD(psDroid, psDroid->player);
 					break;
 				}
-			case DROID_PERSON:
-				// no extra mounts for people
-				break;
-			default:
-				ASSERT(!"invalid droid type", "Whoa! Weirdy type of droid found in drawComponentObject!!!");
+				/*	Get the mounting graphic - we've already moved to the right position
+				Allegedly - all droids will have a mount graphic so this shouldn't
+				fall on it's arse......*/
+				//sensor and cyborg and ecm uses connectors[0]
+				pie_MatBegin(!bButton);
+				/* vtol weapons inverted */
+				if (iConnector >= VTOL_CONNECTOR_START)
+				{
+					pie_MatRotZ(65536 / 2); //this might affect gun rotation
+				}
+
+				pie_TRANSLATE(psShapeTemp->connectors[0].x,
+				              psShapeTemp->connectors[0].z,
+				              psShapeTemp->connectors[0].y);
+
+				pie_MatRotY(-rot.direction);
+				/* Draw it */
+				if (psMountShape)
+				{
+					pie_Draw3DShape(psMountShape, 0, colour, brightness, pieFlag, iPieData);
+				}
+
+				/* translate for construct mount point if cyborg */
+				if (cyborgDroid(psDroid) && psMountShape && psMountShape->nconnectors)
+				{
+					pie_TRANSLATE(psMountShape->connectors[0].x,
+					              psMountShape->connectors[0].z,
+					              psMountShape->connectors[0].y);
+				}
+
+				/* Draw it */
+				if (psShape)
+				{
+					pie_Draw3DShape(psShape, 0, colour, brightness, pieFlag, iPieData);
+
+					// In repair droid case only:
+					if ((psDroid->droidType == DROID_REPAIR || psDroid->droidType == DROID_CYBORG_REPAIR) &&
+					    psShape->nconnectors && psDroid->action == DACTION_DROIDREPAIR)
+					{
+						Spacetime st = interpolateObjectSpacetime(psDroid, graphicsTime);
+
+						pie_TRANSLATE(psShape->connectors[0].x,
+						              psShape->connectors[0].z,
+						              psShape->connectors[0].y);
+						pie_TRANSLATE(0, -20, 0);
+
+						psShape = getImdFromIndex(MI_FLAME);
+
+						/* Rotate for droid */
+						pie_MatRotY(st.rot.direction);
+						pie_MatRotX(-st.rot.pitch);
+						pie_MatRotZ(-st.rot.roll);
+						//rotate Y
+						pie_MatRotY(rot.direction);
+
+						pie_MatRotY(-player.r.y);
+						pie_MatRotX(-player.r.x);
+
+						pie_Draw3DShape(psShape, getModularScaledGraphicsTime(psShape->animInterval, psShape->numFrames), 0, brightness, pie_ADDITIVE, 140);
+
+						pie_MatRotX(player.r.x);
+						pie_MatRotY(player.r.y);
+					}
+				}
+				/* Pop Matrix */
+				pie_MatEnd();
 				break;
 			}
+		case DROID_PERSON:
+			// no extra mounts for people
+			break;
+		default:
+			ASSERT(!"invalid droid type", "Whoa! Weirdy type of droid found in drawComponentObject!!!");
+			break;
 		}
-		/*	We've also got a handle on the psShape here for the weapon which has a connector to point to
-			muzzle flash attachment points - just grab it from psShape->connectors->[x|y|z] */
-	} // end of illustrative indentation - see above
+	}
+	/*	We've also got a handle on the psShape here for the weapon which has a connector to point to
+		muzzle flash attachment points - just grab it from psShape->connectors->[x|y|z] */
 
 	/* set default components transparent */
-	if (psDroid->asBits[COMP_PROPULSION].nStat == 0)
+	if (psDroid->asBits[COMP_PROPULSION] == 0)
 	{
 		pieFlag  |= pie_TRANSLUCENT;
 		iPieData  = DEFAULT_COMPONENT_TRANSLUCENCY;
@@ -957,10 +788,9 @@ static void displayCompObj(DROID *psDroid, bool bButton)
 
 // Render a composite droid given a DROID_TEMPLATE structure.
 //
-void displayComponentButtonTemplate(DROID_TEMPLATE *psTemplate, Vector3i *Rotation, Vector3i *Position, bool RotXYZ, SDWORD scale)
+void displayComponentButtonTemplate(DROID_TEMPLATE *psTemplate, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
-	setMatrix(Position, Rotation, RotXYZ);
-	pie_MatScale(scale / 100.f);
+	setMatrix(Position, Rotation, scale);
 
 	// Decide how to sort it.
 	leftFirst = angleDelta(DEG(Rotation->y)) < 0;
@@ -981,19 +811,18 @@ void displayComponentButtonTemplate(DROID_TEMPLATE *psTemplate, Vector3i *Rotati
 
 // Render a composite droid given a DROID structure.
 //
-void displayComponentButtonObject(DROID *psDroid, Vector3i *Rotation, Vector3i *Position, bool RotXYZ, SDWORD scale)
+void displayComponentButtonObject(DROID *psDroid, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	SDWORD		difference;
 
-	setMatrix(Position, Rotation, RotXYZ);
-	pie_MatScale(scale / 100.f);
+	setMatrix(Position, Rotation, scale);
 
-// Decide how to sort it.
+	// Decide how to sort it.
 	difference = Rotation->y % 360;
 
 	leftFirst = !((difference > 0 && difference < 180) || difference < -180);
 
-// And render the composite object.
+	// And render the composite object.
 	//draw multi component object as a button object
 	displayCompObj(psDroid, true);
 
@@ -1001,7 +830,6 @@ void displayComponentButtonObject(DROID *psDroid, Vector3i *Rotation, Vector3i *
 }
 
 
-
 /* Assumes matrix context is already set */
 // multiple turrets display removed the pointless mountRotation
 void displayComponentObject(DROID *psDroid)
@@ -1012,7 +840,7 @@ void displayComponentObject(DROID *psDroid)
 	leftFirst = angleDelta(player.r.y - st.rot.direction) <= 0;
 
 	/* Push the matrix */
-	pie_MatBegin();
+	pie_MatBegin(true);
 
 	/* Get the real position */
 	position.x = st.pos.x - player.p.x;
@@ -1054,7 +882,7 @@ void displayComponentObject(DROID *psDroid)
 		addEffect(&position, EFFECT_EXPLOSION, EXPLOSION_TYPE_PLASMA, false, NULL, 0);
 	}
 
-	if ((psDroid->visible[selectedPlayer] == UBYTE_MAX) || demoGetStatus())
+	if (psDroid->visible[selectedPlayer] == UBYTE_MAX)
 	{
 		//ingame not button object
 		//should render 3 mounted weapons now
