@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -53,17 +53,22 @@ extern void addPower(int player, int32_t quantity);
 
 void usePower(int player, uint32_t quantity);
 
-/** Update current power based on what was extracted during the last cycle and what Power Generators exist. */
-extern void updatePlayerPower(UDWORD player);
+/** Update current power based on what was extracted during the last cycle and what Power Generators exist.
+  * If ticks is set, this is the number of game ticks to process for at once. */
+extern void updatePlayerPower(int player, int ticks = 1);
 
 /** Used in multiplayer to force power levels. */
 void setPower(unsigned player, int32_t power);
 void setPrecisePower(unsigned player, int64_t power);
 
+void setPowerModifier(int player, int modifier);
+void setPowerMaxStorage(int player, int max);
+
 /** Get the amount of power current held by the given player. */
 int32_t getPower(unsigned player);
 int64_t getPrecisePower(unsigned player);
 int32_t getPowerMinusQueued(unsigned player);
+int getQueuedPower(int player);
 
 /** Resets the power levels for all players when power is turned back on. */
 void powerCalc(bool on);
