@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -57,6 +57,7 @@ bool rotateRadar; ///< Rotate the radar?
 static PIELIGHT		colRadarAlly, colRadarMe, colRadarEnemy;
 static PIELIGHT		tileColours[MAX_TILES];
 static UDWORD		*radarBuffer = NULL;
+static Vector3i		playerpos;
 
 PIELIGHT clanColours[] =
 {
@@ -107,10 +108,11 @@ static float RadarZoomMultiplier = 1.0f;
 static UDWORD radarBufferSize = 0;
 static int frameSkip = 0;
 
-static void DrawRadarTiles(void);
-static void DrawRadarObjects(void);
-static void DrawRadarExtras(float radarX, float radarY, float pixSizeH, float pixSizeV);
-static void DrawNorth(void);
+static void DrawRadarTiles();
+static void DrawRadarObjects();
+static void DrawRadarExtras();
+static void DrawNorth();
+static void setViewingWindow();
 
 static void radarSize(int ZoomLevel)
 {
@@ -130,16 +132,17 @@ static void radarSize(int ZoomLevel)
 	debug(LOG_WZ, "radar=(%u,%u) tex=(%u,%u) size=(%u,%u)", radarCenterX, radarCenterY, radarTexWidth, radarTexHeight, radarWidth, radarHeight);
 }
 
-void radarInitVars(void)
+void radarInitVars()
 {
 	radarTexWidth = 0;
 	radarTexHeight = 0;
 	RadarZoom = DEFAULT_RADARZOOM;
 	debug(LOG_WZ, "Resetting radar zoom to %u", RadarZoom);
 	radarSize(RadarZoom);
+	playerpos = Vector3i(-1, -1, -1);
 }
 
-bool InitRadar(void)
+bool InitRadar()
 {
 	// Ally/enemy/me colors
 	colRadarAlly = WZCOL_YELLOW;
@@ -149,12 +152,10 @@ bool InitRadar(void)
 	{
 		RadarZoom = pie_GetVideoBufferWidth() <= 640 ? 14 : DEFAULT_RADARZOOM * 2;
 	}
-	pie_InitRadar();
-
 	return true;
 }
 
-bool resizeRadar(void)
+bool resizeRadar()
 {
 	if (radarBuffer)
 	{
@@ -164,16 +165,10 @@ bool resizeRadar(void)
 	radarTexHeight = scrollMaxY - scrollMinY;
 	radarBufferSize = radarTexWidth * radarTexHeight * sizeof(UDWORD);
 	radarBuffer = (uint32_t *)malloc(radarBufferSize);
-	if (radarBuffer == NULL)
-	{
-		debug(LOG_FATAL, "Out of memory!");
-		abort();
-		return false;
-	}
 	memset(radarBuffer, 0, radarBufferSize);
 	if (rotateRadar)
 	{
-		RadarZoomMultiplier = (float)MAX(RADWIDTH, RADHEIGHT) / (float)MAX(radarTexWidth, radarTexHeight);
+		RadarZoomMultiplier = (float)std::max(RADWIDTH, RADHEIGHT) / std::max({radarTexWidth, radarTexHeight, 1});
 	}
 	else
 	{
@@ -181,17 +176,17 @@ bool resizeRadar(void)
 	}
 	debug(LOG_WZ, "Setting radar zoom to %u", RadarZoom);
 	radarSize(RadarZoom);
+	pie_SetRadar(-radarWidth / 2.0 - 1, -radarHeight / 2.0 - 1, radarWidth, radarHeight,
+	             radarTexWidth, radarTexHeight, rotateRadar || (RadarZoom % 16 != 0));
+	setViewingWindow();
 
 	return true;
 }
 
-bool ShutdownRadar(void)
+bool ShutdownRadar()
 {
-	pie_ShutdownRadar();
-
 	free(radarBuffer);
 	radarBuffer = NULL;
-
 	return true;
 }
 
@@ -213,9 +208,10 @@ void SetRadarZoom(uint8_t ZoomLevel)
 	RadarZoom = ZoomLevel;
 	radarSize(RadarZoom);
 	frameSkip = 0;
+	resizeRadar();
 }
 
-uint8_t GetRadarZoom(void)
+uint8_t GetRadarZoom()
 {
 	return RadarZoom;
 }
@@ -223,8 +219,8 @@ uint8_t GetRadarZoom(void)
 /** Calculate the radar pixel sizes. Returns pixels per tile. */
 static void CalcRadarPixelSize(float *SizeH, float *SizeV)
 {
-	*SizeH = (float)radarHeight / (float)radarTexHeight;
-	*SizeV = (float)radarWidth / (float)radarTexWidth;
+	*SizeH = (float)radarHeight / std::max(radarTexHeight, 1);
+	*SizeV = (float)radarWidth / std::max(radarTexWidth, 1);
 }
 
 /** Given a position within the radar, return a world coordinate. */
@@ -262,28 +258,26 @@ void CalcRadarPosition(int mX, int mY, int *PosX, int *PosY)
 	*PosY = sPosY;
 }
 
-void drawRadar(void)
+void drawRadar()
 {
 	float	pixSizeH, pixSizeV;
 
-	ASSERT(radarBuffer, "No radar buffer allocated");
-	if (!radarBuffer)
-	{
-		return;
-	}
-
 	CalcRadarPixelSize(&pixSizeH, &pixSizeV);
 
+	ASSERT_OR_RETURN(, radarBuffer, "No radar buffer allocated");
+
+	// Do not recalculate frustum window coordinates if position or zoom does not change
+	if (playerpos.x != player.p.x || playerpos.y != player.p.y || playerpos.z != player.p.z)
+	{
+		setViewingWindow();
+	}
+	playerpos = player.p; // cache position
+
 	if (frameSkip <= 0)
 	{
-		bool filter = true;
-		if (!rotateRadar)
-		{
-			filter = RadarZoom % 16 != 0;
-		}
 		DrawRadarTiles();
 		DrawRadarObjects();
-		pie_DownLoadRadar(radarBuffer, radarTexWidth, radarTexHeight, filter);
+		pie_DownLoadRadar(radarBuffer);
 		frameSkip = RADAR_FRAME_SKIP;
 	}
 	frameSkip--;
@@ -296,24 +290,16 @@ void drawRadar(void)
 		pie_MatRotZ(player.r.y);
 		DrawNorth();
 	}
-	// draw the box at the dimensions of the map
-	iV_TransBoxFill(-radarWidth / 2.0 - 1,
-	                -radarHeight / 2.0 - 1,
-	                radarWidth / 2.0,
-	                radarHeight / 2.0);
-	pie_RenderRadar(-radarWidth / 2.0 - 1,
-	                -radarHeight / 2.0 - 1,
-	                radarWidth,
-	                radarHeight);
+	pie_RenderRadar();
 	pie_MatBegin();
 	pie_TRANSLATE(-radarWidth / 2 - 1, -radarHeight / 2 - 1, 0);
-	DrawRadarExtras(0, 0, pixSizeH, pixSizeV);
+	DrawRadarExtras();
 	pie_MatEnd();
 	drawRadarBlips(-radarWidth / 2.0 - 1, -radarHeight / 2.0 - 1, pixSizeH, pixSizeV);
 	pie_MatEnd();
 }
 
-static void DrawNorth(void)
+static void DrawNorth()
 {
 	iV_DrawImage(IntImages, RADAR_NORTH, -((radarWidth / 2.0) + iV_GetImageWidth(IntImages, RADAR_NORTH) + 1), -(radarHeight / 2.0));
 }
@@ -408,7 +394,7 @@ static PIELIGHT appliedRadarColour(RADAR_DRAW_MODE radarDrawMode, MAPTILE *WTile
 }
 
 /** Draw the map tiles on the radar. */
-static void DrawRadarTiles(void)
+static void DrawRadarTiles()
 {
 	SDWORD	x, y;
 
@@ -420,13 +406,18 @@ static void DrawRadarTiles(void)
 			size_t pos = radarTexWidth * (y - scrollMinY) + (x - scrollMinX);
 
 			ASSERT(pos * sizeof(*radarBuffer) < radarBufferSize, "Buffer overrun");
+			if (y == scrollMinY || x == scrollMinX || y == scrollMaxY - 1 || x == scrollMaxX - 1)
+			{
+				radarBuffer[pos] = WZCOL_BLACK.rgba;
+				continue;
+			}
 			radarBuffer[pos] = appliedRadarColour(radarDrawMode, psTile).rgba;
 		}
 	}
 }
 
 /** Draw the droids and structure positions on the radar. */
-static void DrawRadarObjects(void)
+static void DrawRadarObjects()
 {
 	UBYTE				clan;
 	PIELIGHT			playerCol;
@@ -469,7 +460,7 @@ static void DrawRadarObjects(void)
 				continue;
 			}
 			if (psDroid->visible[selectedPlayer]
-			    || (bMultiPlayer && game.alliance == ALLIANCES_TEAMS
+			    || (bMultiPlayer && alliancesSharedVision(game.alliance)
 			        && aiCheckAlliances(selectedPlayer, psDroid->player)))
 			{
 				int	x = psDroid->pos.x / TILE_UNITS;
@@ -526,7 +517,7 @@ static void DrawRadarObjects(void)
 			flashCol = flashColours[getPlayerColour(clan)];
 
 			if (psStruct->visible[selectedPlayer]
-			    || (bMultiPlayer && game.alliance == ALLIANCES_TEAMS
+			    || (bMultiPlayer && alliancesSharedVision(game.alliance)
 			        && aiCheckAlliances(selectedPlayer, psStruct->player)))
 			{
 				if (clan == selectedPlayer && gameTime - psStruct->timeLastHit < HIT_NOTIFICATION)
@@ -593,13 +584,17 @@ static SDWORD getLengthAdjust(void)
 }
 
 /** Draws a Myth/FF7 style viewing window */
-static void drawViewingWindow(float radarX, float radarY, int x, int y, float pixSizeH, float pixSizeV)
+static void setViewingWindow()
 {
+	float pixSizeH, pixSizeV;
 	Vector3i v[4], tv[4], centre;
 	int	shortX, longX, yDrop, yDropVar;
 	int	dif = getDistanceAdjust();
 	int	dif2 = getLengthAdjust();
 	PIELIGHT colour;
+	CalcRadarPixelSize(&pixSizeH, &pixSizeV);
+	int x = player.p.x * pixSizeH / TILE_UNITS;
+	int y = player.p.z * pixSizeV / TILE_UNITS;
 
 	shortX = ((visibleTiles.x / 4) - (dif / 6)) * pixSizeH;
 	longX = ((visibleTiles.x / 2) - (dif / 4)) * pixSizeH;
@@ -618,8 +613,8 @@ static void drawViewingWindow(float radarX, float radarY, int x, int y, float pi
 	v[3].x = -shortX;
 	v[3].y = yDrop;
 
-	centre.x = radarX + x - scrollMinX * pixSizeH;
-	centre.y = radarY + y - scrollMinY * pixSizeV;
+	centre.x = x - scrollMinX * pixSizeH;
+	centre.y = y - scrollMinY * pixSizeV;
 
 	RotateVector2D(v, tv, &centre, player.r.y, 4);
 
@@ -644,20 +639,16 @@ static void drawViewingWindow(float radarX, float radarY, int x, int y, float pi
 		colour.rgba = 0;
 		colour.byte.a = 0x3f;
 		break;
-
 	}
 
 	/* Send the four points to the draw routine and the clip box params */
-	pie_DrawViewingWindow(tv, radarX, radarY, radarX + radarWidth, radarY + radarHeight, colour);
+	pie_SetViewingWindow(tv, colour);
 }
 
-static void DrawRadarExtras(float radarX, float radarY, float pixSizeH, float pixSizeV)
+static void DrawRadarExtras()
 {
-	int viewX = player.p.x * pixSizeH / TILE_UNITS;
-	int viewY = player.p.z * pixSizeV / TILE_UNITS;
-
-	drawViewingWindow(radarX, radarY, viewX, viewY, pixSizeH, pixSizeV);
-	RenderWindowFrame(FRAME_RADAR, radarX - 1, radarY - 1, radarWidth + 2, radarHeight + 2);
+	pie_DrawViewingWindow();
+	RenderWindowFrame(FRAME_RADAR, -1, -1, radarWidth + 2, radarHeight + 2);
 }
 
 /** Does a screen coordinate lie within the radar area? */
