@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -51,9 +51,7 @@ enum _objids
 	OBJID_SELECTED,		// if droid is selected (humans only)
 	OBJID_TARGET,		// added object->psTarget
 	OBJID_GROUP,		// group a droid belongs to
-	WEAPID_SHORT_RANGE,		// short range of a weapon
 	WEAPID_LONG_RANGE,		// short range of a weapon
-	WEAPID_SHORT_HIT,		// weapon's chance to hit in the short range
 	WEAPID_LONG_HIT,		// weapon's chance to hit in the long range
 	WEAPID_FIRE_PAUSE,		// weapon's fire pause
 	WEAPID_RELOAD_TIME,		// weapon's reload time
