@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 #include "lib/ivis_opengl/piedef.h"
 #include "lib/ivis_opengl/piestate.h"
 #include "lib/ivis_opengl/pieclip.h"
+#include "lib/ivis_opengl/piestate.h"
 #include "lib/ivis_opengl/screen.h"
 
 #include "terrain.h"
@@ -87,20 +88,6 @@ struct DecalVertex
 	GLfloat u, v;          // uv
 };
 
-static struct TerrainData
-{
-	RenderVertex *geometry;
-	RenderVertex *water;
-	DecalVertex *decaldata;
-	PIELIGHT *texture;
-	GLuint *geometryIndex;
-	GLuint *waterIndex;
-	GLuint *textureIndex;
-} terr_terrainData;
-
-/// Use VBO-based terrain renderer
-static bool terr_useVBO = true;
-
 /// The lightmap texture
 static GLuint lightmap_tex_num;
 /// When are we going to update the lightmap next?
@@ -135,28 +122,14 @@ static int xSectors, ySectors;
 /// Did we initialise the terrain renderer yet?
 static bool terrainInitalised = false;
 
-#ifdef DEBUG
-/// Check for OpenGL errors
-#define glError() { \
-		GLenum err = glGetError(); \
-		while (err != GL_NO_ERROR) { \
-			debug(LOG_ERROR, "OpenGL error %i caught at %s:%u\n", (int)err, __FILE__, __LINE__); \
-			err = glGetError(); \
-		} \
-	}
-#else
-#define glError()
-#endif
-
 /// Helper to specify the offset in a VBO
 #define BUFFER_OFFSET(i) ((char *)NULL + (i))
 
 /// Helper variables for the DrawRangeElements functions
-static GLuint dreStart, dreEnd, dreOffset;
-static GLsizei dreCount;
-static GLuint *dreIndexArray;
+GLuint dreStart, dreEnd, dreOffset;
+GLsizei dreCount;
 /// Are we actually drawing something using the DrawRangeElements functions?
-static bool drawRangeElementsStarted = false;
+bool drawRangeElementsStarted = false;
 
 /// Pass all remaining triangles to OpenGL
 static void finishDrawRangeElements(void)
@@ -165,25 +138,12 @@ static void finishDrawRangeElements(void)
 	{
 		ASSERT(dreEnd - dreStart + 1 <= GLmaxElementsVertices, "too many vertices (%i)", (int)(dreEnd - dreStart + 1));
 		ASSERT(dreCount <= GLmaxElementsIndices, "too many indices (%i)", (int)dreCount);
-
-		if (terr_useVBO)
-		{
-			glDrawRangeElements(GL_TRIANGLES,
-			                    dreStart,
-			                    dreEnd,
-			                    dreCount,
-			                    GL_UNSIGNED_INT,
-			                    BUFFER_OFFSET(sizeof(GLuint)*dreOffset));
-		}
-		else
-		{
-			glDrawRangeElements(GL_TRIANGLES,
-			                    dreStart,
-			                    dreEnd,
-			                    dreCount,
-			                    GL_UNSIGNED_INT,
-			                    &dreIndexArray[dreOffset]);
-		}
+		glDrawRangeElements(GL_TRIANGLES,
+		                    dreStart,
+		                    dreEnd,
+		                    dreCount,
+		                    GL_UNSIGNED_INT,
+		                    BUFFER_OFFSET(sizeof(GLuint)*dreOffset));
 	}
 	drawRangeElementsStarted = false;
 }
@@ -197,8 +157,7 @@ static void addDrawRangeElements(GLenum mode,
                                  GLuint end,
                                  GLsizei count,
                                  GLenum type,
-                                 GLuint offset,
-                                 GLuint *indexArray)
+                                 GLuint offset)
 {
 	ASSERT(mode == GL_TRIANGLES, "not supported");
 	ASSERT(type == GL_UNSIGNED_INT, "not supported");
@@ -218,7 +177,6 @@ static void addDrawRangeElements(GLenum mode,
 		dreEnd    = end;
 		dreCount  = count;
 		dreOffset = offset;
-		dreIndexArray = indexArray;
 		drawRangeElementsStarted = true;
 		return;
 	}
@@ -231,7 +189,7 @@ static void addDrawRangeElements(GLenum mode,
 	{
 		finishDrawRangeElements();
 		// start anew
-		addDrawRangeElements(mode, start, end, count, type, offset, indexArray);
+		addDrawRangeElements(mode, start, end, count, type, offset);
 	}
 	else
 	{
@@ -249,6 +207,7 @@ PIELIGHT getTileColour(int x, int y)
 {
 	return mapTile(x, y)->colour;
 }
+
 /// Set the colour of the tile at the specified position
 void setTileColour(int x, int y, PIELIGHT colour)
 {
@@ -429,36 +388,6 @@ static inline void averageColour(PIELIGHT *average, PIELIGHT a, PIELIGHT b,
 	average->byte.b = (a.byte.b + b.byte.b + c.byte.b + d.byte.b) / 4;
 }
 
-/// Get the tiles colour
-static inline void getColour(PIELIGHT *colour, int x, int y, bool center)
-{
-	MAPTILE *psTile;
-
-	if (center)
-	{
-		PIELIGHT a, b, c, d;
-		getColour(&a, x  , y  , false);
-		getColour(&b, x + 1, y  , false);
-		getColour(&c, x  , y + 1, false);
-		getColour(&d, x + 1, y + 1, false);
-		averageColour(colour, a, b, c, d);
-		return;
-	}
-
-	*colour = WZCOL_BLACK;
-	if (x < 0 || y < 0 || x > mapWidth - 1 || y > mapHeight - 1)
-	{
-		return;
-	}
-	psTile = mapTile(x, y);
-	*colour = psTile->colour;
-
-	if (psTile->tileInfoBits & BITS_GATEWAY && showGateways)
-	{
-		colour->byte.g = 255;
-	}
-}
-
 /**
  * Set the terrain and water geometry for the specified sector
  */
@@ -630,76 +559,48 @@ static void setSectorDecals(int x, int y,
  */
 static void updateSectorGeometry(int x, int y)
 {
+	RenderVertex *geometry;
+	RenderVertex *water;
+	DecalVertex *decaldata;
 	int geometrySize = 0;
 	int waterSize = 0;
 	int decalSize = 0;
 
-	if (terr_useVBO)
+	geometry  = (RenderVertex *)malloc(sizeof(RenderVertex) * sectors[x * ySectors + y].geometrySize);
+	water     = (RenderVertex *)malloc(sizeof(RenderVertex) * sectors[x * ySectors + y].waterSize);
+
+	setSectorGeometry(x, y, geometry, water, &geometrySize, &waterSize);
+	ASSERT(geometrySize == sectors[x * ySectors + y].geometrySize, "something went seriously wrong updating the terrain");
+	ASSERT(waterSize    == sectors[x * ySectors + y].waterSize   , "something went seriously wrong updating the terrain");
+
+	glBindBuffer(GL_ARRAY_BUFFER, geometryVBO); glErrors();
+	glBufferSubData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*sectors[x * ySectors + y].geometryOffset,
+	                sizeof(RenderVertex)*sectors[x * ySectors + y].geometrySize, geometry); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, waterVBO); glErrors();
+	glBufferSubData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*sectors[x * ySectors + y].waterOffset,
+	                sizeof(RenderVertex)*sectors[x * ySectors + y].waterSize, water); glErrors();
+
+	free(geometry);
+	free(water);
+
+	if (sectors[x * ySectors + y].decalSize <= 0)
 	{
-		RenderVertex *geometry;
-		RenderVertex *water;
-		DecalVertex *decaldata;
-
-		geometry  = (RenderVertex *)malloc(sizeof(RenderVertex) * sectors[x * ySectors + y].geometrySize);
-		water     = (RenderVertex *)malloc(sizeof(RenderVertex) * sectors[x * ySectors + y].waterSize);
-
-		setSectorGeometry(x, y, geometry, water, &geometrySize, &waterSize);
-
-		ASSERT(geometrySize == sectors[x * ySectors + y].geometrySize, "something went seriously wrong updating the terrain");
-		ASSERT(waterSize    == sectors[x * ySectors + y].waterSize   , "something went seriously wrong updating the terrain");
-
-		glBindBuffer(GL_ARRAY_BUFFER, geometryVBO); glError();
-		glBufferSubData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*sectors[x * ySectors + y].geometryOffset,
-		                sizeof(RenderVertex)*sectors[x * ySectors + y].geometrySize, geometry); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, waterVBO); glError();
-		glBufferSubData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*sectors[x * ySectors + y].waterOffset,
-		                sizeof(RenderVertex)*sectors[x * ySectors + y].waterSize, water); glError();
-
-		free(geometry);
-		free(water);
-
-		if (sectors[x * ySectors + y].decalSize <= 0)
-		{
-			// Nothing to do here, and glBufferSubData(GL_ARRAY_BUFFER, 0, 0, *) crashes in my graphics driver. Probably shouldn't crash...
-			return;
-		}
-
-		decaldata = (DecalVertex *)malloc(sizeof(DecalVertex) * sectors[x * ySectors + y].decalSize);
-
-		setSectorDecals(x, y, decaldata, &decalSize);
-
-		ASSERT(decalSize == sectors[x * ySectors + y].decalSize   , "the amount of decals has changed");
-
-		if (terr_useVBO)
-		{
-			glBindBuffer(GL_ARRAY_BUFFER, decalVBO); glError();
-			glBufferSubData(GL_ARRAY_BUFFER, sizeof(DecalVertex)*sectors[x * ySectors + y].decalOffset,
-			                sizeof(DecalVertex)*sectors[x * ySectors + y].decalSize, decaldata); glError();
-			glBindBuffer(GL_ARRAY_BUFFER, 0);
-		}
-
-		free(decaldata);
+		// Nothing to do here, and glBufferSubData(GL_ARRAY_BUFFER, 0, 0, *) crashes in my graphics driver. Probably shouldn't crash...
+		return;
 	}
-	else
-	{
-		setSectorGeometry(x, y,
-		                  terr_terrainData.geometry + sectors[x * ySectors + y].geometryOffset,
-		                  terr_terrainData.water + sectors[x * ySectors + y].waterOffset,
-		                  &geometrySize, &waterSize);
 
-		ASSERT(geometrySize == sectors[x * ySectors + y].geometrySize, "something went seriously wrong updating the terrain");
-		ASSERT(waterSize    == sectors[x * ySectors + y].waterSize   , "something went seriously wrong updating the terrain");
+	decaldata = (DecalVertex *)malloc(sizeof(DecalVertex) * sectors[x * ySectors + y].decalSize);
+	setSectorDecals(x, y, decaldata, &decalSize);
+	ASSERT(decalSize == sectors[x * ySectors + y].decalSize   , "the amount of decals has changed");
 
-		if (sectors[x * ySectors + y].decalSize <= 0)
-		{
-			// Nothing to do here
-			return;
-		}
+	glBindBuffer(GL_ARRAY_BUFFER, decalVBO); glErrors();
+	glBufferSubData(GL_ARRAY_BUFFER, sizeof(DecalVertex)*sectors[x * ySectors + y].decalOffset,
+	                sizeof(DecalVertex)*sectors[x * ySectors + y].decalSize, decaldata); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 
-		setSectorDecals(x, y, terr_terrainData.decaldata + sectors[x * ySectors + y].decalOffset, &decalSize);
+	free(decaldata);
 
-		ASSERT(decalSize == sectors[x * ySectors + y].decalSize   , "the amount of decals has changed");
-	}
+	glBindBuffer(GL_ARRAY_BUFFER, 0);  // HACK Must unbind GL_ARRAY_BUFFER (don't know if it has to be unbound everywhere), otherwise text rendering may mysteriously crash.
 }
 
 /**
@@ -756,16 +657,20 @@ bool initTerrain(void)
 	PIELIGHT colour[2][2], centerColour;
 	int layer = 0;
 
+	RenderVertex *geometry;
+	RenderVertex *water;
+	DecalVertex *decaldata;
 	int geometrySize, geometryIndexSize;
 	int waterSize, waterIndexSize;
 	int textureSize, textureIndexSize;
+	GLuint *geometryIndex;
+	GLuint *waterIndex;
+	GLuint *textureIndex;
+	PIELIGHT *texture;
 	int decalSize;
-
 	int maxSectorSizeIndices, maxSectorSizeVertices;
 	bool decreasedSize = false;
 
-	debug(LOG_TERRAIN, "VBOs are%s available for usage", terr_useVBO ? "" : "n't");
-
 	// this information is useful to prevent crashes with buggy opengl implementations
 	glGetIntegerv(GL_MAX_ELEMENTS_VERTICES, &GLmaxElementsVertices);
 	glGetIntegerv(GL_MAX_ELEMENTS_INDICES,  &GLmaxElementsIndices);
@@ -820,20 +725,17 @@ bool initTerrain(void)
 	ySectors = (mapHeight + sectorSize - 1) / sectorSize;
 	sectors = (Sector *)malloc(sizeof(Sector) * xSectors * ySectors);
 
-	debug(LOG_TERRAIN, "xSectors: %d, ySectors: %d", xSectors, ySectors);
-
 	////////////////////
 	// fill the geometry part of the sectors
-	terr_terrainData.geometry = (RenderVertex *)malloc(sizeof(RenderVertex) * xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2);
-	terr_terrainData.geometryIndex = (GLuint *)malloc(sizeof(GLuint) * xSectors * ySectors * sectorSize * sectorSize * 12);
+	geometry = (RenderVertex *)malloc(sizeof(RenderVertex) * xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2);
+	geometryIndex = (GLuint *)malloc(sizeof(GLuint) * xSectors * ySectors * sectorSize * sectorSize * 12);
 	geometrySize = 0;
 	geometryIndexSize = 0;
 
-	terr_terrainData.water = (RenderVertex *)malloc(sizeof(RenderVertex) * xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2);
-	terr_terrainData.waterIndex = (GLuint *)malloc(sizeof(GLuint) * xSectors * ySectors * sectorSize * sectorSize * 12);
+	water = (RenderVertex *)malloc(sizeof(RenderVertex) * xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2);
+	waterIndex = (GLuint *)malloc(sizeof(GLuint) * xSectors * ySectors * sectorSize * sectorSize * 12);
 	waterSize = 0;
 	waterIndexSize = 0;
-
 	for (x = 0; x < xSectors; x++)
 	{
 		for (y = 0; y < ySectors; y++)
@@ -844,11 +746,10 @@ bool initTerrain(void)
 			sectors[x * ySectors + y].waterOffset = waterSize;
 			sectors[x * ySectors + y].waterSize = 0;
 
-			setSectorGeometry(x, y, terr_terrainData.geometry, terr_terrainData.water, &geometrySize, &waterSize);
+			setSectorGeometry(x, y, geometry, water, &geometrySize, &waterSize);
 
 			sectors[x * ySectors + y].geometrySize = geometrySize - sectors[x * ySectors + y].geometryOffset;
 			sectors[x * ySectors + y].waterSize = waterSize - sectors[x * ySectors + y].waterOffset;
-
 			// and do the index buffers
 			sectors[x * ySectors + y].geometryIndexOffset = geometryIndexSize;
 			sectors[x * ySectors + y].geometryIndexSize = 0;
@@ -877,42 +778,39 @@ bool initTerrain(void)
 					 */
 #define q(i,j,center) ((x*ySectors+y)*(sectorSize+1)*(sectorSize+1)*2 + ((i)*(sectorSize+1)+(j))*2+(center))
 					// First triangle
-					terr_terrainData.geometryIndex[geometryIndexSize + 0]  = q(i  , j  , 1);	// Center vertex
-					terr_terrainData.geometryIndex[geometryIndexSize + 1]  = q(i  , j  , 0);	// Bottom left
-					terr_terrainData.geometryIndex[geometryIndexSize + 2]  = q(i + 1, j  , 0);	// Bottom right
+					geometryIndex[geometryIndexSize + 0]  = q(i  , j  , 1);	// Center vertex
+					geometryIndex[geometryIndexSize + 1]  = q(i  , j  , 0);	// Bottom left
+					geometryIndex[geometryIndexSize + 2]  = q(i + 1, j  , 0);	// Bottom right
 					// Second triangle
-					terr_terrainData.geometryIndex[geometryIndexSize + 3]  = q(i  , j  , 1);	// Center vertex
-					terr_terrainData.geometryIndex[geometryIndexSize + 4]  = q(i  , j + 1, 0);	// Top left
-					terr_terrainData.geometryIndex[geometryIndexSize + 5]  = q(i  , j  , 0);	// Bottom left
+					geometryIndex[geometryIndexSize + 3]  = q(i  , j  , 1);	// Center vertex
+					geometryIndex[geometryIndexSize + 4]  = q(i  , j + 1, 0);	// Top left
+					geometryIndex[geometryIndexSize + 5]  = q(i  , j  , 0);	// Bottom left
 					// Third triangle
-					terr_terrainData.geometryIndex[geometryIndexSize + 6]  = q(i  , j  , 1);	// Center vertex
-					terr_terrainData.geometryIndex[geometryIndexSize + 7]  = q(i + 1, j + 1, 0);	// Top right
-					terr_terrainData.geometryIndex[geometryIndexSize + 8]  = q(i  , j + 1, 0);	// Top left
+					geometryIndex[geometryIndexSize + 6]  = q(i  , j  , 1);	// Center vertex
+					geometryIndex[geometryIndexSize + 7]  = q(i + 1, j + 1, 0);	// Top right
+					geometryIndex[geometryIndexSize + 8]  = q(i  , j + 1, 0);	// Top left
 					// Fourth triangle
-					terr_terrainData.geometryIndex[geometryIndexSize + 9]  = q(i  , j  , 1);	// Center vertex
-					terr_terrainData.geometryIndex[geometryIndexSize + 10] = q(i + 1, j  , 0);	// Bottom right
-					terr_terrainData.geometryIndex[geometryIndexSize + 11] = q(i + 1, j + 1, 0);	// Top right
-
+					geometryIndex[geometryIndexSize + 9]  = q(i  , j  , 1);	// Center vertex
+					geometryIndex[geometryIndexSize + 10] = q(i + 1, j  , 0);	// Bottom right
+					geometryIndex[geometryIndexSize + 11] = q(i + 1, j + 1, 0);	// Top right
 					geometryIndexSize += 12;
-
 					if (isWater(i + x * sectorSize, j + y * sectorSize))
 					{
-						terr_terrainData.waterIndex[waterIndexSize + 0]  = q(i  , j  , 1);
-						terr_terrainData.waterIndex[waterIndexSize + 1]  = q(i  , j  , 0);
-						terr_terrainData.waterIndex[waterIndexSize + 2]  = q(i + 1, j  , 0);
+						waterIndex[waterIndexSize + 0]  = q(i  , j  , 1);
+						waterIndex[waterIndexSize + 1]  = q(i  , j  , 0);
+						waterIndex[waterIndexSize + 2]  = q(i + 1, j  , 0);
 
-						terr_terrainData.waterIndex[waterIndexSize + 3]  = q(i  , j  , 1);
-						terr_terrainData.waterIndex[waterIndexSize + 4]  = q(i  , j + 1, 0);
-						terr_terrainData.waterIndex[waterIndexSize + 5]  = q(i  , j  , 0);
+						waterIndex[waterIndexSize + 3]  = q(i  , j  , 1);
+						waterIndex[waterIndexSize + 4]  = q(i  , j + 1, 0);
+						waterIndex[waterIndexSize + 5]  = q(i  , j  , 0);
 
-						terr_terrainData.waterIndex[waterIndexSize + 6]  = q(i  , j  , 1);
-						terr_terrainData.waterIndex[waterIndexSize + 7]  = q(i + 1, j + 1, 0);
-						terr_terrainData.waterIndex[waterIndexSize + 8]  = q(i  , j + 1, 0);
-
-						terr_terrainData.waterIndex[waterIndexSize + 9]  = q(i  , j  , 1);
-						terr_terrainData.waterIndex[waterIndexSize + 10] = q(i + 1, j  , 0);
-						terr_terrainData.waterIndex[waterIndexSize + 11] = q(i + 1, j + 1, 0);
+						waterIndex[waterIndexSize + 6]  = q(i  , j  , 1);
+						waterIndex[waterIndexSize + 7]  = q(i + 1, j + 1, 0);
+						waterIndex[waterIndexSize + 8]  = q(i  , j + 1, 0);
 
+						waterIndex[waterIndexSize + 9]  = q(i  , j  , 1);
+						waterIndex[waterIndexSize + 10] = q(i + 1, j  , 0);
+						waterIndex[waterIndexSize + 11] = q(i + 1, j + 1, 0);
 						waterIndexSize += 12;
 					}
 				}
@@ -921,43 +819,34 @@ bool initTerrain(void)
 			sectors[x * ySectors + y].waterIndexSize = waterIndexSize - sectors[x * ySectors + y].waterIndexOffset;
 		}
 	}
+	glGenBuffers(1, &geometryVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, geometryVBO); glErrors();
+	glBufferData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*geometrySize, geometry, GL_DYNAMIC_DRAW); glErrors();
+	free(geometry);
 
-	if (terr_useVBO)
-	{
-		glGenBuffers(1, &geometryVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, geometryVBO); glError();
-		glBufferData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*geometrySize, terr_terrainData.geometry, GL_DYNAMIC_DRAW); glError();
-		free(terr_terrainData.geometry);
-		terr_terrainData.geometry = NULL;
+	glGenBuffers(1, &geometryIndexVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, geometryIndexVBO); glErrors();
+	glBufferData(GL_ARRAY_BUFFER, sizeof(GLuint)*geometryIndexSize, geometryIndex, GL_STATIC_DRAW); glErrors();
+	free(geometryIndex);
 
-		glGenBuffers(1, &geometryIndexVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, geometryIndexVBO); glError();
-		glBufferData(GL_ARRAY_BUFFER, sizeof(GLuint)*geometryIndexSize, terr_terrainData.geometryIndex, GL_STATIC_DRAW); glError();
-		free(terr_terrainData.geometryIndex);
-		terr_terrainData.geometryIndex = NULL;
+	glGenBuffers(1, &waterVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, waterVBO); glErrors();
+	glBufferData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*waterSize, water, GL_DYNAMIC_DRAW); glErrors();
+	free(water);
 
-		glGenBuffers(1, &waterVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, waterVBO); glError();
-		glBufferData(GL_ARRAY_BUFFER, sizeof(RenderVertex)*waterSize, terr_terrainData.water, GL_DYNAMIC_DRAW); glError();
-		free(terr_terrainData.water);
-		terr_terrainData.water = NULL;
+	glGenBuffers(1, &waterIndexVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, waterIndexVBO); glErrors();
+	glBufferData(GL_ARRAY_BUFFER, sizeof(GLuint)*waterIndexSize, waterIndex, GL_STATIC_DRAW); glErrors();
+	free(waterIndex);
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 
-		glGenBuffers(1, &waterIndexVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, waterIndexVBO); glError();
-		glBufferData(GL_ARRAY_BUFFER, sizeof(GLuint)*waterIndexSize, terr_terrainData.waterIndex, GL_STATIC_DRAW); glError();
-		free(terr_terrainData.waterIndex);
-		terr_terrainData.waterIndex = NULL;
-
-		glBindBuffer(GL_ARRAY_BUFFER, 0);
-	}
 
 	////////////////////
 	// fill the texture part of the sectors
-	terr_terrainData.texture = (PIELIGHT *)malloc(sizeof(PIELIGHT) * xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * numGroundTypes);
-	terr_terrainData.textureIndex = (GLuint *)malloc(sizeof(GLuint) * xSectors * ySectors * sectorSize * sectorSize * 12 * numGroundTypes);
+	texture = (PIELIGHT *)malloc(sizeof(PIELIGHT) * xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * numGroundTypes);
+	textureIndex = (GLuint *)malloc(sizeof(GLuint) * xSectors * ySectors * sectorSize * sectorSize * 12 * numGroundTypes);
 	textureSize = 0;
 	textureIndexSize = 0;
-
 	for (layer = 0; layer < numGroundTypes; layer++)
 	{
 		for (x = 0; x < xSectors; x++)
@@ -1023,29 +912,27 @@ bool initTerrain(void)
 								}
 							}
 						}
-						terr_terrainData.texture[xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * layer + ((x * ySectors + y) * (sectorSize + 1) * (sectorSize + 1) * 2 + (i * (sectorSize + 1) + j) * 2)].rgba = colour[0][0].rgba;
+						texture[xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * layer + ((x * ySectors + y) * (sectorSize + 1) * (sectorSize + 1) * 2 + (i * (sectorSize + 1) + j) * 2)].rgba = colour[0][0].rgba;
 						averageColour(&centerColour, colour[0][0], colour[0][1], colour[1][0], colour[1][1]);
-						terr_terrainData.texture[xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * layer + ((x * ySectors + y) * (sectorSize + 1) * (sectorSize + 1) * 2 + (i * (sectorSize + 1) + j) * 2 + 1)].rgba = centerColour.rgba;
+						texture[xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * layer + ((x * ySectors + y) * (sectorSize + 1) * (sectorSize + 1) * 2 + (i * (sectorSize + 1) + j) * 2 + 1)].rgba = centerColour.rgba;
 						textureSize += 2;
-
 						if ((draw) && i < sectorSize && j < sectorSize)
 						{
-							terr_terrainData.textureIndex[textureIndexSize + 0]  = q(i  , j  , 1);
-							terr_terrainData.textureIndex[textureIndexSize + 1]  = q(i  , j  , 0);
-							terr_terrainData.textureIndex[textureIndexSize + 2]  = q(i + 1, j  , 0);
+							textureIndex[textureIndexSize + 0]  = q(i  , j  , 1);
+							textureIndex[textureIndexSize + 1]  = q(i  , j  , 0);
+							textureIndex[textureIndexSize + 2]  = q(i + 1, j  , 0);
 
-							terr_terrainData.textureIndex[textureIndexSize + 3]  = q(i  , j  , 1);
-							terr_terrainData.textureIndex[textureIndexSize + 4]  = q(i  , j + 1, 0);
-							terr_terrainData.textureIndex[textureIndexSize + 5]  = q(i  , j  , 0);
+							textureIndex[textureIndexSize + 3]  = q(i  , j  , 1);
+							textureIndex[textureIndexSize + 4]  = q(i  , j + 1, 0);
+							textureIndex[textureIndexSize + 5]  = q(i  , j  , 0);
 
-							terr_terrainData.textureIndex[textureIndexSize + 6]  = q(i  , j  , 1);
-							terr_terrainData.textureIndex[textureIndexSize + 7]  = q(i + 1, j + 1, 0);
-							terr_terrainData.textureIndex[textureIndexSize + 8]  = q(i  , j + 1, 0);
-
-							terr_terrainData.textureIndex[textureIndexSize + 9]  = q(i  , j  , 1);
-							terr_terrainData.textureIndex[textureIndexSize + 10] = q(i + 1, j  , 0);
-							terr_terrainData.textureIndex[textureIndexSize + 11] = q(i + 1, j + 1, 0);
+							textureIndex[textureIndexSize + 6]  = q(i  , j  , 1);
+							textureIndex[textureIndexSize + 7]  = q(i + 1, j + 1, 0);
+							textureIndex[textureIndexSize + 8]  = q(i  , j + 1, 0);
 
+							textureIndex[textureIndexSize + 9]  = q(i  , j  , 1);
+							textureIndex[textureIndexSize + 10] = q(i + 1, j  , 0);
+							textureIndex[textureIndexSize + 11] = q(i + 1, j + 1, 0);
 							textureIndexSize += 12;
 						}
 
@@ -1056,51 +943,36 @@ bool initTerrain(void)
 			}
 		}
 	}
+	glGenBuffers(1, &textureVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, textureVBO); glErrors();
+	glBufferData(GL_ARRAY_BUFFER, sizeof(PIELIGHT)*xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * numGroundTypes, texture, GL_STATIC_DRAW); glErrors();
+	free(texture);
 
-	if (terr_useVBO)
-	{
-		glGenBuffers(1, &textureVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, textureVBO); glError();
-		glBufferData(GL_ARRAY_BUFFER, sizeof(PIELIGHT)*xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * numGroundTypes, terr_terrainData.texture, GL_STATIC_DRAW); glError();
-		free(terr_terrainData.texture);
-		terr_terrainData.texture = NULL;
-
-		glGenBuffers(1, &textureIndexVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, textureIndexVBO); glError();
-		glBufferData(GL_ARRAY_BUFFER, sizeof(GLuint)*textureIndexSize, terr_terrainData.textureIndex, GL_STATIC_DRAW); glError();
-		free(terr_terrainData.textureIndex);
-		terr_terrainData.textureIndex = NULL;
-
-		glBindBuffer(GL_ARRAY_BUFFER, 0);
-	}
+	glGenBuffers(1, &textureIndexVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, textureIndexVBO); glErrors();
+	glBufferData(GL_ARRAY_BUFFER, sizeof(GLuint)*textureIndexSize, textureIndex, GL_STATIC_DRAW); glErrors();
+	free(textureIndex);
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 
 	// and finally the decals
-	terr_terrainData.decaldata = (DecalVertex *)malloc(sizeof(DecalVertex) * mapWidth * mapHeight * 12);
+	decaldata = (DecalVertex *)malloc(sizeof(DecalVertex) * mapWidth * mapHeight * 12);
 	decalSize = 0;
-
 	for (x = 0; x < xSectors; x++)
 	{
 		for (y = 0; y < ySectors; y++)
 		{
 			sectors[x * ySectors + y].decalOffset = decalSize;
 			sectors[x * ySectors + y].decalSize = 0;
-			setSectorDecals(x, y, terr_terrainData.decaldata, &decalSize);
+			setSectorDecals(x, y, decaldata, &decalSize);
 			sectors[x * ySectors + y].decalSize = decalSize - sectors[x * ySectors + y].decalOffset;
 		}
 	}
-
 	debug(LOG_TERRAIN, "%i decals found", decalSize / 12);
-
-	if (terr_useVBO)
-	{
-		glGenBuffers(1, &decalVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, decalVBO); glError();
-		glBufferData(GL_ARRAY_BUFFER, sizeof(DecalVertex)*decalSize, terr_terrainData.decaldata, GL_STATIC_DRAW); glError();
-		free(terr_terrainData.decaldata);
-		terr_terrainData.decaldata = NULL;
-
-		glBindBuffer(GL_ARRAY_BUFFER, 0);
-	}
+	glGenBuffers(1, &decalVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, decalVBO); glErrors();
+	glBufferData(GL_ARRAY_BUFFER, sizeof(DecalVertex)*decalSize, decaldata, GL_STATIC_DRAW); glErrors();
+	free(decaldata);
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 
 	lightmap_tex_num = 0;
 	lightmapLastUpdate = 0;
@@ -1125,8 +997,8 @@ bool initTerrain(void)
 	glBindTexture(GL_TEXTURE_2D, lightmap_tex_num);
 
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 
@@ -1134,6 +1006,8 @@ bool initTerrain(void)
 
 	terrainInitalised = true;
 
+	glBindBuffer(GL_ARRAY_BUFFER, 0);  // HACK Must unbind GL_ARRAY_BUFFER (in this function, at least), otherwise text rendering may mysteriously crash.
+
 	return true;
 }
 
@@ -1141,34 +1015,13 @@ bool initTerrain(void)
 void shutdownTerrain(void)
 {
 	ASSERT_OR_RETURN(, sectors, "trying to shutdown terrain when it didn't need it!");
-
-	if (terr_useVBO)
-	{
-		glDeleteBuffers(1, &geometryVBO);
-		glDeleteBuffers(1, &geometryIndexVBO);
-		glDeleteBuffers(1, &waterVBO);
-		glDeleteBuffers(1, &waterIndexVBO);
-		glDeleteBuffers(1, &textureVBO);
-		glDeleteBuffers(1, &textureIndexVBO);
-		glDeleteBuffers(1, &decalVBO);
-	}
-	else
-	{
-		free(terr_terrainData.geometry);
-		terr_terrainData.geometry = NULL;
-		free(terr_terrainData.geometryIndex);
-		terr_terrainData.geometryIndex = NULL;
-		free(terr_terrainData.water);
-		terr_terrainData.water = NULL;
-		free(terr_terrainData.waterIndex);
-		terr_terrainData.waterIndex = NULL;
-		free(terr_terrainData.texture);
-		terr_terrainData.texture = NULL;
-		free(terr_terrainData.textureIndex);
-		terr_terrainData.textureIndex = NULL;
-		free(terr_terrainData.decaldata);
-		terr_terrainData.decaldata = NULL;
-	}
+	glDeleteBuffers(1, &geometryVBO);
+	glDeleteBuffers(1, &geometryIndexVBO);
+	glDeleteBuffers(1, &waterVBO);
+	glDeleteBuffers(1, &waterIndexVBO);
+	glDeleteBuffers(1, &textureVBO);
+	glDeleteBuffers(1, &textureIndexVBO);
+	glDeleteBuffers(1, &decalVBO);
 
 	for (int x = 0; x < xSectors; x++)
 	{
@@ -1206,7 +1059,7 @@ void drawTerrain(void)
 	const GLfloat paramsY[4] = {0, 0, -1.0f / world_coord(mapHeight) *((float)mapHeight / lightmapHeight), 0};
 
 	///////////////////////////////////
-	glError();	// clear error codes
+	glErrors();	// clear error codes
 	// set up the lightmap texture
 	glActiveTexture(GL_TEXTURE1);
 	// bind the texture
@@ -1222,7 +1075,18 @@ void drawTerrain(void)
 		{
 			for (i = 0; i < mapWidth; ++i)
 			{
-				const PIELIGHT colour = getTileColour(i, j);
+				MAPTILE *psTile = mapTile(i, j);
+				PIELIGHT colour = psTile->colour;
+
+				if (psTile->tileInfoBits & BITS_GATEWAY && showGateways)
+				{
+					colour.byte.g = 255;
+				}
+				if (psTile->tileInfoBits & BITS_MARKED)
+				{
+					int m = getModularScaledGraphicsTime(2048, 255);
+					colour.byte.r = MAX(m, 255 - m);
+				}
 
 				lightmapPixmap[(i + j * lightmapWidth) * 3 + 0] = colour.byte.r;
 				lightmapPixmap[(i + j * lightmapWidth) * 3 + 1] = colour.byte.g;
@@ -1304,8 +1168,8 @@ void drawTerrain(void)
 	}
 
 	// enable texture coord generation
-	glEnable(GL_TEXTURE_GEN_S); glError();
-	glEnable(GL_TEXTURE_GEN_T); glError();
+	glEnable(GL_TEXTURE_GEN_S); glErrors();
+	glEnable(GL_TEXTURE_GEN_T); glErrors();
 
 	// set the parameters for the texture coord generation
 	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
@@ -1345,17 +1209,10 @@ void drawTerrain(void)
 
 	// bind the vertex buffer
 	glEnableClientState(GL_VERTEX_ARRAY);
+	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, geometryIndexVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, geometryVBO); glErrors();
 
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, geometryIndexVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, geometryVBO); glError();
-		glVertexPointer(3, GL_FLOAT, sizeof(RenderVertex), BUFFER_OFFSET(0)); glError();
-	}
-	else
-	{
-		glVertexPointer(3, GL_FLOAT, 0, terr_terrainData.geometry); glError();
-	}
+	glVertexPointer(3, GL_FLOAT, sizeof(RenderVertex), BUFFER_OFFSET(0)); glErrors();
 
 	for (x = 0; x < xSectors; x++)
 	{
@@ -1368,18 +1225,13 @@ void drawTerrain(void)
 				                     sectors[x * ySectors + y].geometryOffset + sectors[x * ySectors + y].geometrySize,
 				                     sectors[x * ySectors + y].geometryIndexSize,
 				                     GL_UNSIGNED_INT,
-				                     sectors[x * ySectors + y].geometryIndexOffset,
-				                     terr_terrainData.geometryIndex);
+				                     sectors[x * ySectors + y].geometryIndexOffset);
 			}
 		}
 	}
 	finishDrawRangeElements();
-
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ARRAY_BUFFER, 0);
-		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
-	}
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
 
 	if (pie_GetFogStatus())
 	{
@@ -1397,6 +1249,8 @@ void drawTerrain(void)
 	///////////////////////////////////
 	// terrain
 
+	glEnableClientState(GL_COLOR_ARRAY);
+
 	// set up for texture coord generation
 	glEnable(GL_TEXTURE_GEN_S);
 	glEnable(GL_TEXTURE_GEN_T);
@@ -1409,22 +1263,14 @@ void drawTerrain(void)
 	// only draw colors
 	glDepthMask(GL_FALSE);
 
-	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-
 	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
+	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, textureIndexVBO); glErrors();
 
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, textureIndexVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, geometryVBO);
-		glVertexPointer(3, GL_FLOAT, sizeof(RenderVertex), BUFFER_OFFSET(0)); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, textureVBO);
-	}
-	else
-	{
-		glVertexPointer(3, GL_FLOAT, 0, terr_terrainData.geometry); glError();
-	}
+	// load the vertex (geometry) buffer
+	glBindBuffer(GL_ARRAY_BUFFER, geometryVBO);
+	glVertexPointer(3, GL_FLOAT, sizeof(RenderVertex), BUFFER_OFFSET(0)); glErrors();
+
+	glBindBuffer(GL_ARRAY_BUFFER, textureVBO);
 
 	ASSERT_OR_RETURN(, psGroundTypes, "Ground type was not set, no textures will be seen.");
 
@@ -1445,14 +1291,7 @@ void drawTerrain(void)
 		glTexGenfv(GL_T, GL_OBJECT_PLANE, paramsY);
 
 		// load the color buffer
-		if (terr_useVBO)
-		{
-			glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(PIELIGHT), BUFFER_OFFSET(sizeof(PIELIGHT)*xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * layer));
-		}
-		else
-		{
-			glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(PIELIGHT), &terr_terrainData.texture[xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * layer]);
-		}
+		glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(PIELIGHT), BUFFER_OFFSET(sizeof(PIELIGHT)*xSectors * ySectors * (sectorSize + 1) * (sectorSize + 1) * 2 * layer));
 
 		for (x = 0; x < xSectors; x++)
 		{
@@ -1465,32 +1304,26 @@ void drawTerrain(void)
 					                     sectors[x * ySectors + y].geometryOffset + sectors[x * ySectors + y].geometrySize,
 					                     sectors[x * ySectors + y].textureIndexSize[layer],
 					                     GL_UNSIGNED_INT,
-					                     sectors[x * ySectors + y].textureIndexOffset[layer],
-					                     terr_terrainData.textureIndex);
+					                     sectors[x * ySectors + y].textureIndexOffset[layer]);
 				}
 			}
 		}
 		finishDrawRangeElements();
 	}
-
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ARRAY_BUFFER, 0);
-		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
-	}
-
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
 	// we don't need this one anymore
 	glDisableClientState(GL_COLOR_ARRAY);
 
 	// no more texture generation
-	glDisable(GL_TEXTURE_GEN_S); glError();
-	glDisable(GL_TEXTURE_GEN_T); glError();
+	glDisable(GL_TEXTURE_GEN_S); glErrors();
+	glDisable(GL_TEXTURE_GEN_T); glErrors();
 
 	//////////////////////////////////
 	// decals
 
 	// select the terrain texture page
-	pie_SetTexturePage(terrainPage); glError();
+	pie_SetTexturePage(terrainPage); glErrors();
 
 	// use the alpha to blend
 	pie_SetRendMode(REND_ALPHA);
@@ -1498,20 +1331,12 @@ void drawTerrain(void)
 	glColor3f(1.f, 1.f, 1.f);
 
 	// and the texture coordinates buffer
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY); glError();
-	glEnableClientState(GL_VERTEX_ARRAY); glError();
-
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ARRAY_BUFFER, decalVBO); glError();
-		glVertexPointer(3, GL_FLOAT, sizeof(DecalVertex), BUFFER_OFFSET(0)); glError();
-		glTexCoordPointer(2, GL_FLOAT, sizeof(DecalVertex), BUFFER_OFFSET(12)); glError();
-	}
-	else
-	{
-		glVertexPointer(3, GL_FLOAT, sizeof(DecalVertex), terr_terrainData.decaldata); glError();
-		glTexCoordPointer(2, GL_FLOAT, sizeof(DecalVertex), &terr_terrainData.decaldata->u); glError();
-	}
+	glEnableClientState(GL_TEXTURE_COORD_ARRAY); glErrors();
+	glEnableClientState(GL_VERTEX_ARRAY); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, decalVBO); glErrors();
+	glVertexPointer(3, GL_FLOAT, sizeof(DecalVertex), BUFFER_OFFSET(0)); glErrors();
+	glTexCoordPointer(2, GL_FLOAT, sizeof(DecalVertex), BUFFER_OFFSET(12)); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 
 	size = 0;
 	offset = 0;
@@ -1528,7 +1353,7 @@ void drawTerrain(void)
 			// can't append, so draw what we have and start anew
 			if (size > 0)
 			{
-				glDrawArrays(GL_TRIANGLES, offset, size); glError();
+				glDrawArrays(GL_TRIANGLES, offset, size); glErrors();
 			}
 			size = 0;
 			if (y < ySectors && sectors[x * ySectors + y].draw)
@@ -1539,11 +1364,6 @@ void drawTerrain(void)
 		}
 	}
 
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ARRAY_BUFFER, 0);
-	}
-
 	////////////////////////////////
 	// disable the lightmap texture
 	glActiveTexture(GL_TEXTURE1);
@@ -1561,6 +1381,8 @@ void drawTerrain(void)
 	glDisableClientState(GL_VERTEX_ARRAY);
 
 	glDepthMask(GL_TRUE);
+
+	//glBindBuffer(GL_ARRAY_BUFFER, 0);  // HACK Must unbind GL_ARRAY_BUFFER (don't know if it has to be unbound everywhere), otherwise text rendering may mysteriously crash.
 }
 
 /**
@@ -1580,8 +1402,8 @@ void drawWater(void)
 	                              pie_GetFogColour().byte.a / 255.f
 	                             };
 
-	glEnable(GL_TEXTURE_GEN_S); glError();
-	glEnable(GL_TEXTURE_GEN_T); glError();
+	glEnable(GL_TEXTURE_GEN_S); glErrors();
+	glEnable(GL_TEXTURE_GEN_T); glErrors();
 
 	glColor4ub(0xFF, 0xFF, 0xFF, 0xFF);
 	glDepthMask(GL_FALSE);
@@ -1592,7 +1414,7 @@ void drawWater(void)
 	}
 
 	// first texture unit
-	pie_SetTexturePage(iV_GetTexture("page-80")); glError();
+	pie_SetTexturePage(iV_GetTexture("page-80-water-1.png")); glErrors();
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
 	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
@@ -1608,12 +1430,11 @@ void drawWater(void)
 
 	// second texture unit
 	glActiveTexture(GL_TEXTURE1);
-	glBindTexture(GL_TEXTURE_2D, iV_NativeTexID(iV_GetTexture("page-81")));
-	glError();
+	glBindTexture(GL_TEXTURE_2D, pie_Texture(iV_GetTexture("page-81-water-2.png")));
+	glErrors();
 	glEnable(GL_TEXTURE_2D);
-	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-	glEnable(GL_TEXTURE_GEN_S); glError();
-	glEnable(GL_TEXTURE_GEN_T); glError();
+	glEnable(GL_TEXTURE_GEN_S); glErrors();
+	glEnable(GL_TEXTURE_GEN_T); glErrors();
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
 	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
@@ -1625,17 +1446,10 @@ void drawWater(void)
 
 	// bind the vertex buffer
 	glEnableClientState(GL_VERTEX_ARRAY);
+	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, waterIndexVBO); glErrors();
+	glBindBuffer(GL_ARRAY_BUFFER, waterVBO); glErrors();
 
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, waterIndexVBO); glError();
-		glBindBuffer(GL_ARRAY_BUFFER, waterVBO); glError();
-		glVertexPointer(3, GL_FLOAT, sizeof(RenderVertex), BUFFER_OFFSET(0)); glError();
-	}
-	else
-	{
-		glVertexPointer(3, GL_FLOAT, 0, terr_terrainData.water); glError();
-	}
+	glVertexPointer(3, GL_FLOAT, sizeof(RenderVertex), BUFFER_OFFSET(0)); glErrors();
 
 	for (x = 0; x < xSectors; x++)
 	{
@@ -1648,18 +1462,13 @@ void drawWater(void)
 				                     sectors[x * ySectors + y].geometryOffset + sectors[x * ySectors + y].geometrySize,
 				                     sectors[x * ySectors + y].waterIndexSize,
 				                     GL_UNSIGNED_INT,
-				                     sectors[x * ySectors + y].waterIndexOffset,
-				                     terr_terrainData.waterIndex);
+				                     sectors[x * ySectors + y].waterIndexOffset);
 			}
 		}
 	}
 	finishDrawRangeElements();
-
-	if (terr_useVBO)
-	{
-		glBindBuffer(GL_ARRAY_BUFFER, 0);
-		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
-	}
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
 
 	glDisableClientState(GL_VERTEX_ARRAY);
 
