@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2013  Warzone 2100 Project
+	Copyright (C) 2005-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -24,9 +24,29 @@
 #ifndef __INCLUDED_WEAPONDEF_H__
 #define __INCLUDED_WEAPONDEF_H__
 
+// maximum difference in direction for a fixed turret to fire
+#define FIXED_TURRET_DIR DEG(1)
+
+// %age at which a unit is considered to be heavily damaged
+#define HEAVY_DAMAGE_LEVEL	25
+
+/* who specified the target? */
+enum TARGET_ORIGIN
+{
+	ORIGIN_UNKNOWN,		///< Default value if unknown
+	ORIGIN_PLAYER,		///< Came directly from player
+	ORIGIN_VISUAL,		///< Visual targeting
+	ORIGIN_ALLY,		///< Came from allied unit/structure
+	ORIGIN_COMMANDER,	///< Came from commander
+	ORIGIN_SENSOR,		///< Came from standard sensor
+	ORIGIN_CB_SENSOR,	///< Came from counter-battery sensor
+	ORIGIN_AIRDEF_SENSOR,	///< Came from Air Defense sensor
+	ORIGIN_RADAR_DETECTOR,	///< Came from Radar Detector sensor
+};
+
 struct WEAPON
 {
-	unsigned int    nStat;		///< Index into the asWeaponStats global array
+	uint32_t        nStat;		///< Index into the asWeaponStats global array
 	uint32_t        ammo;
 	uint32_t        lastFired;	///< The gametime when this weapon last fired
 	uint32_t        shotsFired;
