@@ -1,6 +1,6 @@
 /*
 	This file is part of Warzone 2100.
-	Copyright (C) 2007-2013  Warzone 2100 Project
+	Copyright (C) 2007-2015  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -184,8 +184,8 @@ static void check_pie(const char *input)
 			}
 			if (flags & iV_IMD_TEXANIM)
 			{
-				int frames, rate, width, height;
-				num = fscanf(fp, "%d %d %d %d", &frames, &rate, &width, &height);
+				int frames, rate; float width, height;
+				num = fscanf(fp, "%d %d %f %f", &frames, &rate, &width, &height);
 				if (num != 4)
 				{
 					fprintf(stderr, "File %s - Bad texture animation entry level %d, number %d.\n", input, level, j);
